//==========================================================================
//
//      devs/eth/arm/epxa1/..../include/devs_eth_epxa1.inl
//
//      EPXA1 ethernet I/O definitions.
//
//==========================================================================

//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    Qiang Huang
// Contributors: None
// Date:         2003-02-19
// Purpose:      EPXA1 ethernet definitions
//####DESCRIPTIONEND####
//==========================================================================

#include <pkgconf/system.h>
#include <pkgconf/devs_eth_altera_epxa1.h>
#include <cyg/hal/hal_intr.h>          
#include <cyg/hal/epxa1.h>

#define CYGNUM_DEVS_ETH_SMSC_LAN91C111_SHIFT_ADDR    0
// MAC address is stored as a Redboot config option
#ifdef CYGPKG_REDBOOT
#include <pkgconf/redboot.h>
#ifdef CYGSEM_REDBOOT_FLASH_CONFIG
#include <redboot.h>
#include <flash_config.h>

RedBoot_config_option("Network hardware address [MAC]",
                      epxa1_esa,
                      ALWAYS_ENABLED, true,
                      CONFIG_ESA, 0
    );
#endif
#endif

#ifndef CYGSEM_DEVS_ETH_ALTERA_EPXA1_REDBOOT_ESA
static unsigned char static_esa[] = CYGDAT_DEVS_ETH_ALTERA_EPXA1_ESA;
#endif

// ESA address fetch function
static void epxa1_get_ESA(struct lan91c111_priv_data *cpd)
{
    // Fetch hardware address from RedBoot config
#if defined(CYGSEM_DEVS_ETH_ALTERA_EPXA1_REDBOOT_ESA)
#if defined(CYGPKG_REDBOOT) && \
    defined(CYGSEM_REDBOOT_FLASH_CONFIG)
    flash_get_config("epxa1_esa", cpd->enaddr, CONFIG_ESA);
#else
#error "No RedBoot flash configuration to store ESA"
#endif
#else
    memcpy(cpd->enaddr, static_esa, 6);
#endif
}

static lan91c111_priv_data lan91c111_eth0_priv_data = { 

    config_enaddr : epxa1_get_ESA,
#ifndef CYGSEM_DEVS_ETH_ALTERA_EPXA1_REDBOOT_ESA    
    enaddr: CYGDAT_DEVS_ETH_ALTERA_EPXA1_ESA,
#endif
    base : (unsigned short *) ALTERA_EPXA1_ETH_IOBASE,
    interrupt : EPXA1_IRQ_GPIO_ETH
};

ETH_DRV_SC(lan91c111_sc,
           &lan91c111_eth0_priv_data,          // Driver specific data
           CYGDAT_DEVS_ETH_ALTERA_EPXA1_NAME, // Name for device
           lan91c111_start,
           lan91c111_stop,
           lan91c111_control,
           lan91c111_can_send,
           lan91c111_send,
           lan91c111_recv,
           lan91c111_deliver,
           lan91c111_poll,
           lan91c111_int_vector
);

NETDEVTAB_ENTRY(lan91c111_netdev, 
                "lan91c111_" CYGDAT_DEVS_ETH_ALTERA_EPXA1_NAME,
                smsc_lan91c111_init,
                &lan91c111_sc);

//EOF devs_eth_epxa1.inl


