#ifndef CYGONCE_DEVS_ETH_SMSC_LAN91C111_LAN91C111_H
#define CYGONCE_DEVS_ETH_SMSC_LAN91C111_LAN91C111_H
//==========================================================================
//
//      lan91c111.h
//
//      SMCS LAN91C111 (LAN91C111 compatible) Ethernet chip
//
//==========================================================================

//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    Qiang Huang
// Contributors: None
// Date:         2003-01-20
// Purpose:      Hardware description of LAN9000 series, LAN91C111.
// Description:  
//
//####DESCRIPTIONEND####
//
//==========================================================================

#include <cyg/hal/hal_io.h>

// ===============================================================================================
// -------------------------------------------  BANK 0 -------------------------------------------
#define LAN91C111_TCR         0x00	// Transmit control register -- Bank 0 -- Offset 0
#define LAN91C111_EPH_STATUS  0x01	// Status of Last transmitted frame -- Bank 0 -- Offset 2
#define LAN91C111_RCR         0x02	// Receive Control Register -- Bank 0 -- Offset 4
#define LAN91C111_COUNTER     0x03	// Counter Register -- Bank 0 -- Offset 6
#define LAN91C111_MIR         0x04	// Memory Information Register -- Bank 0 -- Offset 8
// ------------------------------------------------------------------------------------------------
// Difference
#define LAN91C111_RPHYCR      0x05	// Receive/Phy Control Register -- Bank 0 -- Offset A
// ------------------------------------------------------------------------------------------------
#define LAN91C111_RESERVED_0  0x06	// Reserved -- Bank 0 -- Offset C
#define LAN91C111_BS          0x07	// Bank Selection Register -- Bank 0 -- Offset E
// ----------------------------------------  END OF BANK 0 ----------------------------------------
// ================================================================================================



// ===============================================================================================
// -------------------------------------------  BANK 1 -------------------------------------------
#define LAN91C111_CONFIG      0x08	// Configuration Register -- Bank 1 -- Offset 0
#define LAN91C111_BASE_REG    0x09	// Base Address Register -- Offset 2
#define LAN91C111_IA01        0x0a	// Individual Address Registers 0,1 -- Offset 4
#define LAN91C111_IA23        0x0b	// Individual Address Registers 2,3 -- Offset 6
#define LAN91C111_IA45        0x0c	// Individual Address Registers 4,5 -- Offset 8
#define LAN91C111_GENERAL     0x0d	// General Purpose Register -- Offset A
#define LAN91C111_CONTROL     0x0e	// Control Register -- Offset C
#define LAN91C111_BS2         0x0f	// Bank Selection Register -- Offset E
// ----------------------------------------  END OF BANK 1 ----------------------------------------
// ================================================================================================



// ===============================================================================================
// -------------------------------------------  BANK 2 -------------------------------------------
#define LAN91C111_MMU_COMMAND 0x10	// MMU Command Register -- Offset 0
#define LAN91C111_PNR         0x11	// Packet Number Register -- Offset 2
#define LAN91C111_FIFO_PORTS  0x12	// FIFO Ports Register -- Offset 4
#define LAN91C111_POINTER     0x13	// Pointer Register -- Offset 6
#define LAN91C111_DATA_HIGH   0x14	// Data Register High -- Offset 8
#define LAN91C111_DATA        0x15	// Data Register Low -- Offset A
#define LAN91C111_INTERRUPT   0x16	// Interrupt Status Registers -- Offset C
#define LAN91C111_BS3         0x17	// Bank Selection Register -- Offset E
// ----------------------------------------  END OF BANK 2 ----------------------------------------
// ================================================================================================



// ===============================================================================================
// -------------------------------------------  BANK 3 -------------------------------------------
#define LAN91C111_MT01        0x18	// Multicast Table Register 0,1 -- Offset 0
#define LAN91C111_MT23        0x19	// Multicast Table Register 2,3 -- Offset 2
#define LAN91C111_MT45        0x1a	// Multicast Table Register 4,5 -- Offset 4
#define LAN91C111_MT67        0x1b	// Multicast Table Register 6,7 -- Offset 6
#define LAN91C111_MGMT        0x1c	// Management Interface -- Offset 8
#define LAN91C111_REVISION    0x1d	// Revision Register -- Offset A
#define LAN91C111_ERCV        0x1e	// Early RCV Register -- Offset C
#define LAN91C111_BS4         0x1f	// Bank Selectioin Register -- Offset E
// ----------------------------------------  END OF BANK 3 ----------------------------------------
// ================================================================================================





// enable auto-neogotiation mode
#define LAN91C111_RPHYCR_ANEG	0x3800	

#define LAN91C111_RCR_SOFT_RST   0x8000    // soft reset
#define LAN91C111_RCR_FILT_CAR   0x4000    // filter carrier
#define LAN91C111_RCR_ABORT_ENB  0x2000    // abort on collision
#define LAN91C111_RCR_STRIP_CRC  0x0200    // strip CRC
#define LAN91C111_RCR_RXEN       0x0100    // enable RX
#define LAN91C111_RCR_ALMUL      0x0004    // receive all muticasts
#define LAN91C111_RCR_PRMS       0x0002    // promiscuous
#define LAN91C111_RCR_RX_ABORT   0x0001    // set when abort due to long frame

#define LAN91C111_TCR_SWFDUP     0x8000    // Switched Full Duplex mode
#define LAN91C111_TCR_ETEN_TYPE  0x4000    // ETEN type (91C96) 0 <=> like a 91C94
#define LAN91C111_TCR_EPH_LOOP   0x2000    // loopback mode
#define LAN91C111_TCR_STP_SQET   0x1000    // Stop transmission on SQET error
#define LAN91C111_TCR_FDUPLX     0x0800    // full duplex
#define LAN91C111_TCR_MON_CSN    0x0400    // monitor carrier during tx (91C96)
#define LAN91C111_TCR_NOCRC      0x0100    // does not append CRC to frames
#define LAN91C111_TCR_PAD_EN     0x0080    // pads frames with 00 to min length
#define LAN91C111_TCR_FORCOL     0x0004    // force collision
#define LAN91C111_TCR_LLOOP      0x0002    // local loopback (91C96)
#define LAN91C111_TCR_TXENA      0x0001    // enable

#define LAN91C111_POINTER_RCV        0x8000
#define LAN91C111_POINTER_AUTO_INCR  0x4000
#define LAN91C111_POINTER_READ       0x2000
#define LAN91C111_POINTER_ETEN       0x1000
#define LAN91C111_POINTER_NOT_EMPTY  0x0800


#define LAN91C111_INTERRUPT_TX_IDLE_M      0x8000 // (91C96)
#define LAN91C111_INTERRUPT_ERCV_INT_M     0x4000
#define LAN91C111_INTERRUPT_EPH_INT_M      0x2000
#define LAN91C111_INTERRUPT_RX_OVRN_INT_M  0x1000
#define LAN91C111_INTERRUPT_ALLOC_INT_M    0x0800
#define LAN91C111_INTERRUPT_TX_EMPTY_INT_M 0x0400
#define LAN91C111_INTERRUPT_TX_INT_M       0x0200
#define LAN91C111_INTERRUPT_RCV_INT_M      0x0100
#define LAN91C111_INTERRUPT_TX_IDLE        0x0080 // (91C96)
#define LAN91C111_INTERRUPT_ERCV_INT       0x0040 // also ack
#define LAN91C111_INTERRUPT_EPH_INT        0x0020
#define LAN91C111_INTERRUPT_RX_OVRN_INT    0x0010 // also ack
#define LAN91C111_INTERRUPT_ALLOC_INT      0x0008
#define LAN91C111_INTERRUPT_TX_EMPTY_INT   0x0004 // also ack
#define LAN91C111_INTERRUPT_TX_INT         0x0002 // also ack
#define LAN91C111_INTERRUPT_RCV_INT        0x0001

#if 0 // Whichever we choose, the behaviour is the same.
#define LAN91C111_INTERRUPT_TX_SET         0x0002 // TX
#define LAN91C111_INTERRUPT_TX_SET_ACK     0x0000 // -none-
#define LAN91C111_INTERRUPT_TX_FIFO_ACK    0x0002 // TX alone
#define LAN91C111_INTERRUPT_TX_SET_M       0x0200 // TX alone
#else
#define LAN91C111_INTERRUPT_TX_SET         0x0006 // TX_EMPTY + TX
#define LAN91C111_INTERRUPT_TX_SET_ACK     0x0004 // TX_EMPTY and not plain TX
#define LAN91C111_INTERRUPT_TX_FIFO_ACK    0x0002 // TX alone
#define LAN91C111_INTERRUPT_TX_SET_M       0x0600 // TX_EMPTY + TX
#endif

#define LAN91C111_CONTROL_RCV_BAD       0x4000
#define LAN91C111_CONTROL_AUTO_RELEASE  0x0800
#define LAN91C111_CONTROL_LE_ENABLE     0x0080
#define LAN91C111_CONTROL_CR_ENABLE     0x0040
#define LAN91C111_CONTROL_TE_ENABLE     0x0020

// These are for setting the MAC address in the 91C96 serial EEPROM
#define LAN91C111_CONTROL_EEPROM_SELECT 0x0004
#define LAN91C111_CONTROL_RELOAD        0x0002
#define LAN91C111_CONTROL_STORE         0x0001
#define LAN91C111_CONTROL_EEPROM_BUSY   0x0003
#define LAN91C111_ESA_EEPROM_OFFSET     0x0020

#define LAN91C111_STATUS_TX_UNRN        0x8000
#define LAN91C111_STATUS_LINK_OK        0x4000
#define LAN91C111_STATUS_CTR_ROL        0x1000
#define LAN91C111_STATUS_EXC_DEF        0x0800
#define LAN91C111_STATUS_LOST_CARR      0x0400
#define LAN91C111_STATUS_LATCOL         0x0200
#define LAN91C111_STATUS_WAKEUP         0x0100
#define LAN91C111_STATUS_TX_DEFR        0x0080
#define LAN91C111_STATUS_LTX_BRD        0x0040
#define LAN91C111_STATUS_SQET           0x0020
#define LAN91C111_STATUS_16COL          0x0010
#define LAN91C111_STATUS_LTX_MULT       0x0008
#define LAN91C111_STATUS_MUL_COL        0x0004
#define LAN91C111_STATUS_SNGL_COL       0x0002
#define LAN91C111_STATUS_TX_SUC         0x0001

#define LAN91C111_MMU_noop              0x0000
#define LAN91C111_MMU_alloc_for_tx      0x0020
#define LAN91C111_MMU_reset_mmu         0x0040
#define LAN91C111_MMU_rem_rx_frame      0x0060
#define LAN91C111_MMU_rem_tx_frame      0x0070 // (91C96) only when TX stopped
#define LAN91C111_MMU_remrel_rx_frame   0x0080
#define LAN91C111_MMU_rel_packet        0x00a0
#define LAN91C111_MMU_enq_packet        0x00c0
#define LAN91C111_MMU_reset_tx_fifo     0x00e0

#define LAN91C111_CONTROLBYTE_CRC       0x1000
#define LAN91C111_CONTROLBYTE_ODD       0x2000
#define LAN91C111_CONTROLBYTE_RX        0x4000

#define LAN91C111_RX_STATUS_ALIGNERR    0x8000
#define LAN91C111_RX_STATUS_BCAST       0x4000
#define LAN91C111_RX_STATUS_BADCRC      0x2000
#define LAN91C111_RX_STATUS_ODDFRM      0x1000
#define LAN91C111_RX_STATUS_TOOLONG     0x0800
#define LAN91C111_RX_STATUS_TOOSHORT    0x0400
#define LAN91C111_RX_STATUS_HASHVALMASK 0x007e // MASK
#define LAN91C111_RX_STATUS_MCAST       0x0001


// Attribute memory registers in PCMCIA mode
#define LAN91C111_ECOR                  0x8000
#define LAN91C111_ECOR_RESET            (1<<7)
#define LAN91C111_ECOR_LEVIRQ           (1<<6)
#define LAN91C111_ECOR_ATTWR            (1<<2)
#define LAN91C111_ECOR_ENABLE           (1<<0)

#define LAN91C111_ECSR                  0x8002
#define LAN91C111_ECSR_IOIS8            (1<<5)
#define LAN91C111_ECSR_PWRDWN           (1<<2)
#define LAN91C111_ECSR_INTR             (1<<1)



// HQ TESTING
#define	RPC_REG		0x000A
#define	RPC_SPEED	0x2000	// When 1 PHY is in 100Mbps mode.
#define	RPC_DPLX	0x1000	// When 1 PHY is in Full-Duplex Mode
#define	RPC_ANEG	0x0800	// When 1 PHY is in Auto-Negotiate Mode
#define	RPC_LSXA_SHFT	5	// Bits to shift LS2A,LS1A,LS0A to lsb
#define	RPC_LSXB_SHFT	2	// Bits to get LS2B,LS1B,LS0B to lsb
#define RPC_LED_100_10	(0x00)	// LED = 100Mbps OR's with 10Mbps link detect
#define RPC_LED_RES	(0x01)	// LED = Reserved
#define RPC_LED_10	(0x02)	// LED = 10Mbps link detect
#define RPC_LED_FD	(0x03)	// LED = Full Duplex Mode
#define RPC_LED_TX_RX	(0x04)	// LED = TX or RX packet occurred
#define RPC_LED_100	(0x05)	// LED = 100Mbps link dectect
#define RPC_LED_TX	(0x06)	// LED = TX packet occurred
#define RPC_LED_RX	(0x07)	// LED = RX packet occurred
#define RPC_DEFAULT (RPC_ANEG | (RPC_LED_100_10 << RPC_LSXA_SHFT) | (RPC_LED_TX_RX << RPC_LSXB_SHFT) | RPC_SPEED | RPC_DPLX)
// Receive Control Register
/* BANK 0  */
#define	RCR_REG		0x0004
#define	RCR_RX_ABORT	0x0001	// Set if a rx frame was aborted
#define	RCR_PRMS	0x0002	// Enable promiscuous mode
#define	RCR_ALMUL	0x0004	// When set accepts all multicast frames
#define RCR_RXEN	0x0100	// IFF this is set, we can receive packets
#define	RCR_STRIP_CRC	0x0200	// When set strips CRC from rx packets
#define	RCR_ABORT_ENB	0x0200	// When set will abort rx on collision 
#define	RCR_FILT_CAR	0x0400	// When set filters leading 12 bit s of carrier
#define RCR_SOFTRST	0x8000 	// resets the chip

/* the normal settings for the RCR register : */
#define	RCR_DEFAULT	(RCR_STRIP_CRC | RCR_RXEN)
#define RCR_CLEAR	0x0	// set it to a base state
// Transmit Control Register
/* BANK 0  */
#define	TCR_REG 	0x0000 	// transmit control register 
#define TCR_ENABLE	0x0001	// When 1 we can transmit
#define TCR_LOOP	0x0002	// Controls output pin LBK
#define TCR_FORCOL	0x0004	// When 1 will force a collision
#define TCR_PAD_EN	0x0080	// When 1 will pad tx frames < 64 bytes w/0
#define TCR_NOCRC	0x0100	// When 1 will not append CRC to tx frames
#define TCR_MON_CSN	0x0400	// When 1 tx monitors carrier
#define TCR_FDUPLX    	0x0800  // When 1 enables full duplex operation
#define TCR_STP_SQET	0x1000	// When 1 stops tx if Signal Quality Error
#define	TCR_EPH_LOOP	0x2000	// When 1 enables EPH block loopback
#define	TCR_SWFDUP	0x8000	// When 1 enables Switched Full Duplex mode

#define	TCR_CLEAR	0	/* do NOTHING */
/* the default settings for the TCR register : */ 
/* QUESTION: do I want to enable padding of short packets ? */
#define	TCR_DEFAULT  	(TCR_ENABLE |TCR_FDUPLX)
// Configuration Reg
/* BANK 1 */
#define CONFIG_REG	0x0000
#define CONFIG_EXT_PHY	0x0200	// 1=external MII, 0=internal Phy
#define CONFIG_GPCNTRL	0x0400	// Inverse value drives pin nCNTRL
#define CONFIG_NO_WAIT	0x1000	// When 1 no extra wait states on ISA bus
#define CONFIG_EPH_POWER_EN 0x8000 // When 0 EPH is placed into low power mode.

// Default is powered-up, Internal Phy, Wait States, and pin nCNTRL=low
#define CONFIG_DEFAULT	(CONFIG_EPH_POWER_EN)

#define MII_MSK_CRS100	0x4000 // Disables CRS100 detection during tx half dup
#define MII_MDOE	0x0008 // MII Output Enable
#define MII_MCLK	0x0004 // MII Clock, pin MDCLK
#define MII_MDI		0x0002 // MII Input, pin MDI
#define MII_MDO		0x0001 // MII Output, pin MDO
// PHY Control Register
#define PHY_CNTL_REG		0x00
#define PHY_CNTL_RST		0x8000	// 1=PHY Reset
#define PHY_CNTL_LPBK		0x4000	// 1=PHY Loopback
#define PHY_CNTL_SPEED		0x2000	// 1=100Mbps, 0=10Mpbs
#define PHY_CNTL_ANEG_EN	0x1000 // 1=Enable Auto negotiation
#define PHY_CNTL_PDN		0x0800	// 1=PHY Power Down mode
#define PHY_CNTL_MII_DIS	0x0400	// 1=MII 4 bit interface disabled
#define PHY_CNTL_ANEG_RST	0x0200 // 1=Reset Auto negotiate
#define PHY_CNTL_DPLX		0x0100	// 1=Full Duplex, 0=Half Duplex
#define PHY_CNTL_COLTST		0x0080	// 1= MII Colision Test

// PHY Identifier Registers
#define PHY_ID1_REG		0x02	// PHY Identifier 1
#define PHY_ID2_REG		0x03	// PHY Identifier 2

void udelay (unsigned int ud)
{
	volatile unsigned int i;
	int cnt;
	
	for(cnt=0; cnt<ud; cnt++)
		for(i=0;i<10000;i++)
			;
}






// ------------------------------------------------------------------------

#ifdef KEEP_STATISTICS
struct smsc_lan91c111_stats {
    unsigned int tx_good             ;
    unsigned int tx_max_collisions   ;
    unsigned int tx_late_collisions  ;
    unsigned int tx_underrun         ;
    unsigned int tx_carrier_loss     ;
    unsigned int tx_deferred         ;
    unsigned int tx_sqetesterrors    ;
    unsigned int tx_single_collisions;
    unsigned int tx_mult_collisions  ;
    unsigned int tx_total_collisions ;
    unsigned int rx_good             ;
    unsigned int rx_crc_errors       ;
    unsigned int rx_align_errors     ;
    unsigned int rx_resource_errors  ;
    unsigned int rx_overrun_errors   ;
    unsigned int rx_collisions       ;
    unsigned int rx_short_frames     ;
    unsigned int rx_too_long_frames  ;
    unsigned int rx_symbol_errors    ;
    unsigned int interrupts          ;
    unsigned int rx_count            ;
    unsigned int rx_deliver          ;
    unsigned int rx_resource         ;
    unsigned int rx_restart          ;
    unsigned int tx_count            ;
    unsigned int tx_complete         ;
    unsigned int tx_dropped          ;
};
#endif

typedef struct lan91c111_priv_data {
    int txbusy;                         // A packet has been sent
    unsigned long txkey;                // Used to ack when packet sent
    unsigned short* base;               // Base I/O address of controller
	unsigned long phyaddr;
                                        // (as it comes out of reset)
#if CYGINT_DEVS_ETH_SMSC_LAN91C111_PCMCIA_MODE					
    unsigned char* attbase;             // Base attribute address of controller
                                        // only used in PCMCIA mode
#endif				
    int interrupt;                      // Interrupt vector used by controller
    unsigned char enaddr[6];            // Controller ESA
    // Function to configure the ESA - may fetch ESA from EPROM or 
    // RedBoot config option.
    void (*config_enaddr)(struct lan91c111_priv_data* cpd);
    int txpacket;
    int rxpacket;
    int within_send;
    int addrsh;                         // Address bits to shift
#ifdef KEEP_STATISTICS
    struct smsc_lan91c111_stats stats;
#endif
} lan91c111_priv_data;

// ------------------------------------------------------------------------

static __inline__ unsigned short
get_reg(struct eth_drv_sc *sc, int regno)
{
    struct lan91c111_priv_data *cpd =
        (struct lan91c111_priv_data *)sc->driver_private;
    unsigned short val;
    
    HAL_WRITE_UINT16(cpd->base+(LAN91C111_BS << cpd->addrsh), regno>>3);
    HAL_READ_UINT16(cpd->base+((regno&0x7) << cpd->addrsh), val);

#if DEBUG & 2
//	diag_printf("write reg address: bank reg: 0x%08x,  reg: 0x%08x\n", cpd->base+(LAN91C111_BS << cpd->addrsh), cpd->base+((regno&0x7) << cpd->addrsh));
    diag_printf("read reg %d val 0x%04x\n", regno, val);
#endif
    return val;
}

static __inline__ void
put_reg(struct eth_drv_sc *sc, int regno, unsigned short val)
{
    struct lan91c111_priv_data *cpd =
        (struct lan91c111_priv_data *)sc->driver_private;
	
    HAL_WRITE_UINT16(cpd->base+(LAN91C111_BS << cpd->addrsh), regno>>3);
    HAL_WRITE_UINT16(cpd->base+((regno&0x7) << cpd->addrsh), val);

#if DEBUG & 2
//	diag_printf("write reg address: bank reg: 0x%08x,  reg: 0x%08x\n", cpd->base+(LAN91C111_BS << cpd->addrsh), cpd->base+((regno&0x7) << cpd->addrsh));
    diag_printf("write reg %d val 0x%04x\n", regno, val);
#endif
}

// ------------------------------------------------------------------------
// Assumes bank2 has been selected
static __inline__ void
put_data(struct eth_drv_sc *sc, unsigned short val)
{
    struct lan91c111_priv_data *cpd =
        (struct lan91c111_priv_data *)sc->driver_private;
	
    HAL_WRITE_UINT16(cpd->base+((LAN91C111_DATA & 0x7) << cpd->addrsh), val);

#if DEBUG & 2
//	diag_printf("write data to address: 0x%08x\n", cpd->base+((LAN91C111_DATA & 0x7) << cpd->addrsh));
    diag_printf("write data 0x%04x\n", val);
#endif
}

// Assumes bank2 has been selected
static __inline__ unsigned short
get_data(struct eth_drv_sc *sc)
{
    unsigned short val;
    struct lan91c111_priv_data *cpd =
        (struct lan91c111_priv_data *)sc->driver_private;
	
    HAL_READ_UINT16(cpd->base+((LAN91C111_DATA & 0x7) << cpd->addrsh), val);

#if DEBUG & 2
//	diag_printf("get data from address: 0x%08x\n", cpd->base+((LAN91C111_DATA & 0x7) << cpd->addrsh));
    diag_printf("read data 0x%04x\n", val);
#endif
    return val;
}

// ------------------------------------------------------------------------
// Read the bank register (this one is bank-independent)
static __inline__ unsigned short
get_banksel(struct eth_drv_sc *sc)
{
    struct lan91c111_priv_data *cpd =
        (struct lan91c111_priv_data *)sc->driver_private;
    unsigned short val;
    
    HAL_READ_UINT16(cpd->base+(LAN91C111_BS << cpd->addrsh), val);
#if DEBUG & 2
//	diag_printf("cpd->base: 0x%08x\n", cpd->base);
//	diag_printf("cpd->addrsh 0x%08x\n", cpd->addrsh);
//	diag_printf("read bank val Address 0x%08x\n", cpd->base+(LAN91C111_BS << cpd->addrsh) );
    diag_printf("read bank val 0x%04x\n", val);
#endif
    return val;
}


// ------------------------------------------------------------------------
// Write on PCMCIA attribute memory
#if CYGINT_DEVS_ETH_SMSC_LAN91C111_PCMCIA_MODE					
static __inline__ void
put_att(struct eth_drv_sc *sc, int offs, unsigned char val)
{
    struct lan91c111_priv_data *cpd =
        (struct lan91c111_priv_data *)sc->driver_private;
	
    HAL_WRITE_UINT8(cpd->attbase + (offs << cpd->addrsh), val);

#if DEBUG & 2
    diag_printf("write attr %d val 0x%02x\n", offs, val);
#endif
}

// Read from PCMCIA attribute memory
static __inline__ unsigned char
get_att(struct eth_drv_sc *sc, int offs)
{
    struct lan91c111_priv_data *cpd =
        (struct lan91c111_priv_data *)sc->driver_private;
    unsigned char val;
    
    HAL_READ_UINT8(cpd->attbase + (offs << cpd->addrsh), val);
#if DEBUG & 2
    diag_printf("read attr %d val 0x%02x\n", offs, val);
#endif
    return val;
}
#endif // #if CYGINT_DEVS_ETH_SMSC_LAN91C111_PCMCIA_MODE					

// ------------------------------------------------------------------------
#endif CYGONCE_DEVS_ETH_SMSC_LAN91C111_LAN91C111_H
// EOF smsc_lan91c111.h
