#ifndef CAN_LIB_H
#define CAN_LIB_H

#include <pkgconf/io.h>
#include <cyg/io/io.h>
#include <cyg/hal/hal_intr.h>
#include <cyg/io/devtab.h>
#include <cyg/infra/diag.h>
#include <cyg/io/def.h>

/* ---------------- TYPES ----------------- */
typedef enum 
{
	TXInt=0,
	RXInt=1,
	OverInt=2,
	ErrInt=3,
	GenSyncRX=4,
	GenSyncTX=5,
	BPR0=6,
	BPR1=7,
	TXDLC0=8,
	TXDLC1=9,
	TXDLC2=10,
	TXDLC3=11,
	TMRMR=12,
	TXEM=13,
	TXRM=14,
	Disabled=15,
	PS1_0=16,
	PS1_1=17,
	PS1_2=18,
	PS1_3=19,
	PS2_0=20,
	PS2_1=21,
	PS2_2=22,
	PS2_3=23,
	RSJ0=24,
	RSJ1=25,
	RSJ2=26,
	TXReq=27,
	AbortTX=28,
	IntClr=29,
	Reset=30,
	RXClr=31
} SETUP_BIT;
	
typedef enum
{
	BusOff= 0,
	ErrPass= 1,
	TXActive= 2,
	TXOk= 3,
	RXOk= 4,
	RXOverrun= 5,
	SyncRX= 6,
	SyncTX= 7,
	RXDLC0= 8,
	RXDLC1= 9,
	RXDLC2= 10,
	RXDLC3= 11,
	RXExtFrame= 12
} STATUS_BIT;

typedef struct 
{
	unsigned short mask;
	unsigned short code;
} FilterType;

typedef struct 
{
	unsigned char byte[8];
} CANBytesType;

typedef unsigned short TriggerMatchType;
typedef cyg_uint16 ArbitrationType;

typedef struct 
{
	ArbitrationType arbitration;
	unsigned char length;
	unsigned char data[8];
} CANMsgType;

typedef struct
{
	unsigned int	std_id;
	unsigned int	ext_id;
	unsigned char	length;
	unsigned char	data[8];
	unsigned char	can_block_mode;
	cyg_flag_t		*pcan_block_flag;

} CANMsgStr;

// function declaration
unsigned int SingleBit(unsigned char i);
void ReqSend(void);
void SetSetupBit(SETUP_BIT bit); 
void ClearSetupBit(SETUP_BIT bit); 
unsigned char StatusBit(STATUS_BIT bit); 
void WriteTXCanMessage(CANMsgType can_type_msg); 
CANMsgType TXCanMessage(void); 
void SetTXMsgLength(unsigned char l);
void WriteFilter(FilterType fil); 
FilterType Filter(void); 
CANMsgType RXMessage(void);
unsigned char RXMsgLength(void);
unsigned char RXErrCnt(void); 
unsigned char TXErrCnt(void); 
void WriteTriggerMatch(TriggerMatchType t); 
TriggerMatchType TriggerMatch(void);
void setCANSpeed(void);
void first_send(void);
void initTX(void);
void first_send2(void);
void initTX2(void);
void initHurryAMBA(void);
void SendStdCANMessage(CANMsgType can_type_msg); 
void ReadCANMessage(CANMsgType *can_type_msg); 
// config the can controller speed
cyg_uint8 CAN_set_speed(cyg_uint8 speed);
// config the can controller speed
cyg_uint8 CAN_set_mask_filter(FilterType mask_filter);
unsigned int get_can_status(void);


#ifdef __cplusplus
extern "C" cyg_uint16 translate_std_id(cyg_uint16 std_id);
extern "C" cyg_uint16 translate_ext_id(cyg_uint16 ext_id);
#else
cyg_uint16 translate_std_id(cyg_uint16 std_id);
cyg_uint16 translate_ext_id(cyg_uint16 ext_id);
#endif

#endif
