#include <cyg/io/can_api.h>

extern cyg_mutex_t mutex_send, mutex_recv;
extern cyg_sem_t semaphore_send;

// can related user api functions definition

// this function is called to send out a CAN message
cyg_uint8 api_can_send(CANMsgStr can_str_msg)
{
	cyg_uint8 flag;

	can_str_msg.can_block_mode = CAN_NOBLOCK;

	// first add the can message to the system buffer
	flag = node_add_send(can_str_msg);

	// should make the send_thread to send out the message
	// increment the counting semorphore, while non-zero then a CAN message will be 
	// sent out by the can_send_thread background thread
	cyg_semaphore_post(&semaphore_send);

	return flag;
}

// function block until the CAN msg successfully sent
cyg_uint8 api_can_send_block(CANMsgStr *can_str_msg)
{
	cyg_uint8 flag;
	cyg_flag_t can_block_flag;


	// init the flag to wait on flag associated with the can message structure
    cyg_flag_init( &can_block_flag);

	can_str_msg->pcan_block_flag = &can_block_flag;

//	diag_printf("Flag address: 0x%08x!\n", can_str_msg->pcan_block_flag );

	// set it to block mode
	can_str_msg->can_block_mode = CAN_BLOCK;

	// add the can message to the system sending buffer list
	flag = node_add_send(*can_str_msg);

	// signal the system can send thread to do the job
	cyg_semaphore_post(&semaphore_send);

	// wait until the flag is set
	cyg_flag_wait( can_str_msg->pcan_block_flag, -1, CYG_FLAG_WAITMODE_OR | CYG_FLAG_WAITMODE_CLR );

//	diag_printf("\n\n\nCongratulation Flag Set!!\n\n\n\n");

	return flag;
}

// get the can msg without being block
cyg_uint8 api_can_recv(CANMsgStr *pcan_str_msg, cyg_uint8 mode)
{
	// try to retrieve the can message from the system buffer with the specified CAN ID
	// mode = MODE_REMOVE, then remove the can msg from the system buffer
	// mode = MODE_NOREMOVE, leave it

	return ( node_get_msg(pcan_str_msg, mode) );
}

// block until the can msg arrived
cyg_uint8 api_can_recv_block(CANMsgStr *pcan_str_msg, cyg_uint8 mode)
{

	node_get_msg_block(pcan_str_msg, mode);
	return CAN_NOERROR;
}

// get the can msg with highest priority
cyg_uint8 api_can_recv_higest_prio(CANMsgStr *pcan_str_msg, cyg_uint8 mode)
{
	CANMsgStr can_str_msg;
	cyg_uint8 cnt, flag;
	flag = node_get_hiprio_can_msg(&can_str_msg, MODE_RECV, mode);
	if(flag == CAN_NOERROR)
	{
		pcan_str_msg->std_id = can_str_msg.std_id;
		pcan_str_msg->ext_id = can_str_msg.ext_id;
		pcan_str_msg->length = can_str_msg.length;
		for(cnt=0; cnt<pcan_str_msg->length; cnt++)
			pcan_str_msg->data[cnt] = can_str_msg.data[cnt];

		pcan_str_msg->can_block_mode = can_str_msg.can_block_mode;
		pcan_str_msg->pcan_block_flag = can_str_msg.pcan_block_flag;
	}

	return flag;	
}

// user api function used to set the speed of the CAN controller
cyg_uint8 api_can_set_speed(cyg_uint8 speed)
{
	return( CAN_set_speed(speed) );
}

// set the acceptance mask and filter
cyg_uint8 api_can_set_mask_filter(cyg_uint16 can_mask, cyg_uint16 can_filter)
{
	FilterType m_filter;

	m_filter.mask = can_mask & 0x7ff;
	m_filter.code = can_filter & 0x7ff;

	// set the can controller only receive the CAN Msg with the specified ID

	return ( CAN_set_mask_filter(m_filter) );

}

cyg_uint8 cyg_api_can_set_speed(cyg_uint8 speed)
{
	cyg_io_handle_t handle;
	cyg_uint8 flag;
	unsigned char buf[]={' ', ' ', ' ', ' ', speed};

	if( cyg_io_lookup(can_controller_name, &handle) == ENOERR)
		flag = (cyg_uint8) cyg_io_set_config(handle, CAN_SPEED, buf, sizeof(buf));
	else
		flag = CAN_ERROR;
	
	return flag;
}

cyg_uint8 cyg_api_can_set_mask_filter(cyg_uint16 can_mask, cyg_uint16 can_filter)
{
	cyg_io_handle_t handle;
	int flag;
	unsigned char buf[]={	((unsigned char)(can_mask&0xff)),   ((unsigned char)( (can_mask>>8)&0xff)),
							((unsigned char)(can_filter&0xff)), ((unsigned char)( (can_filter>>8)&0xff)), 
							' ' };

	if( cyg_io_lookup(can_controller_name, &handle) == ENOERR)
		flag = (cyg_uint8) cyg_io_set_config(handle, CAN_MASK_FILTER, buf, sizeof(buf));
	else
		flag = CAN_ERROR;
	
	return flag;
}

cyg_uint8 cyg_api_can_get_config(cyg_uint8 buf[])
{
	cyg_io_handle_t handle;
	int flag;

	if( cyg_io_lookup(can_controller_name, &handle) == ENOERR ) 
		flag = (cyg_uint8) cyg_io_get_config(handle, 0, buf, 5);
	else
		flag = CAN_ERROR;

	return flag;

}

