// ====================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    Qiang Huang
// Contributors: None
// Date:         2003-04-11
// Purpose:      Soft CAN device driver
// Description:
//
//####DESCRIPTIONEND####
// ====================================================================

#include <cyg/io/can_drv.h>

static soft_can_channel  soft_can_channel0 ={	interrupt : CYGNUM_HAL_INTERRUPT_PLD_5,
												mask : 0x00,
												filter : 0x00, 
												speed : CAN_500K };



static cyg_interrupt soft_can_interrupt;
static cyg_handle_t  soft_can_interrupt_handle;
unsigned long time_stamp=0;
cyg_uint8 can_tx=0;
cyg_uint8 can_rx=0;

// here is the name 
unsigned char can_controller_name[]="soft_can_name";

// stacks for CAN receive background thread and send background thread
static char can_recv_stack[4096], can_send_stack[4096];

// send and receive thread per-thread data
static cyg_thread can_recv_thread_data, can_send_thread_data;

// handlers for can send and receive thread
static cyg_handle_t can_recv_thread_handle, can_send_thread_handle;

// flag for signalling the receiving thread for a can message arrived
cyg_flag_t can_recv_flag;

// flag for successfully transmitting CAN message
cyg_flag_t can_send_flag;  

// mutex for send and receive threads
cyg_mutex_t mutex_send, mutex_recv;

// semaphore for counting the events of the send command from the user application
cyg_sem_t semaphore_send;

// if using SOPC then activate the PIO interrupt support
void set_pio(void)
{
	// make the port as input
	na_can_pio->np_piodirection = 0x00;
	// enable the pio interrupt
	na_can_pio->np_piointerruptmask = 0x1;
}

// define CAN I/O functions
// CAN I/O function for writing
Cyg_ErrNo soft_can_write(cyg_io_handle_t handle, const void *buf, cyg_uint32 *len, cyg_uint32 pos)
{
	return ENOERR;
}

// CAN I/O function for reading
Cyg_ErrNo soft_can_read(cyg_io_handle_t handle, void *buf, cyg_uint32 *len, cyg_uint32 pos)
{
	return ENOERR;
}

// CAN I/O function for selection
cyg_bool  soft_can_select(cyg_io_handle_t handle, cyg_uint32 which, CYG_ADDRWORD info)
{
	return ENOERR;
}

// CAN I/O function for retrieving the configuration information
// retrieved info in buf[]
// buf[0,1,2,3] = CAN mask, filter
// buf[0] = mask[0~7]
// buf[1] = mask[8~15]
// buf[2] = filter[16~23]
// buf[3] = filter[24~31]
// buf[4] = CAN speed
Cyg_ErrNo soft_can_get_config(cyg_io_handle_t handle, cyg_uint32 key, void *buf, cyg_uint32 *len)
{
	unsigned char *pbuf = buf;
    cyg_devtab_entry_t *t = (cyg_devtab_entry_t *)handle;
    soft_can_channel *can_channel = (soft_can_channel *)t->priv;

//	diag_printf(" Get Config Called!\n");

	// get mask
	pbuf[0] = can_channel->mask & 0xff;
	pbuf[1] = (can_channel->mask>>8) & 0xff;

	// get filter
	pbuf[2] = can_channel->filter & 0xff;
	pbuf[3] = (can_channel->filter>>8) & 0xff;

	// get speed
	pbuf[4] = can_channel->speed;
	
	return CAN_NOERROR;
}


// buf[0,1,2,3] = CAN mask, filter
// buf[0] = mask[0~7]
// buf[1] = mask[8~15]
// buf[2] = filter[16~23]
// buf[3] = filter[24~31]
// buf[4] = CAN speed

// CAN I/O function for setting the configuration information
Cyg_ErrNo soft_can_set_config(cyg_io_handle_t handle, cyg_uint32 key, const void *buf, cyg_uint32 *len)
{
	int flag;
	unsigned char *pbuf = buf;
	FilterType mask_filter;

    cyg_devtab_entry_t *t = (cyg_devtab_entry_t *)handle;
    soft_can_channel *can_channel = (soft_can_channel *)t->priv;

	if(key == CAN_SPEED)
	{
//		diag_printf("Config called Set CAN Speed!\n");
		flag = (int) CAN_set_speed(pbuf[4]);
		if(flag != CAN_ERROR)
			can_channel->speed = pbuf[4];
	}

	else if(key == CAN_MASK_FILTER)
	{
//		diag_printf("Config called Set CAN Mask Filter!\n");

		mask_filter.mask = ( (pbuf[0] | (pbuf[1]<<8)) & 0x7ff );
		mask_filter.code = ( (pbuf[2] | (pbuf[3]<<8)) & 0x7ff );
		flag = (int)CAN_set_mask_filter(mask_filter);	
		if(flag != CAN_ERROR)
		{
			can_channel->mask = mask_filter.mask;
			can_channel->filter = mask_filter.code;
		}
	}
	else
		flag = CAN_ERROR;

	return flag;
}

// setup a soft_can device I/O function table
// this is a block mode table
BLOCK_DEVIO_TABLE(soft_can_io_handler_table,soft_can_write,soft_can_read,soft_can_select,soft_can_get_config,soft_can_set_config)

// soft can controller ISR
// This ISR is called when the soft can controller (PLD) interrupt occurs
static int soft_can_isr(cyg_vector_t vector, cyg_addrword_t data)
{
   soft_can_channel *sc = (soft_can_channel *)data;

  
	// display the CAN status
//  diag_printf("IRQ CAN Status: 0x%04x!\n", get_can_status());

	// TX or RX Interrupt
   
	// if RX IRQ then set the flag and let the DSR to finish the job
	if( StatusBit(RXOk) )
		can_rx = CAN_RX_IRQ;
	else
		can_rx = CAN_NONE_IRQ;

	// if TX IRQ then set the flag and let the DSR to finish the job
	if( StatusBit(TXOk) )
	{
		can_tx = CAN_TX_IRQ;
	}
	else
		can_tx = CAN_NONE_IRQ;
	
	// if any error occured then reset the CAN controller to clear the error condiction
	if( (!StatusBit(TXOk)) && (!StatusBit(RXOk)) )
	{
		diag_printf("CAN Controller Error Occured!\n");
		SetSetupBit(Reset);
		can_tx = CAN_TX_IRQ;
	}
	
	// clear soft can controller interrupt of the CAN controller
	SetSetupBit(IntClr);
	// clear the PLD IRQ 5 of the EPXA1
    cyg_interrupt_acknowledge(sc->interrupt);

	// Run the DSR
    return (CYG_ISR_HANDLED|CYG_ISR_CALL_DSR);  
}


// soft can controller DSR
void soft_can_dsr(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data)
{

	// if the RX IRQ occured
	if(can_rx == CAN_RX_IRQ)
		// Signal the can background thread to receive the data from CAN device
		cyg_flag_setbits( &can_recv_flag, 1 ); 

	// if TX IRQ occured
	if(can_tx == CAN_TX_IRQ)
		// signal the transmission is finished
		cyg_flag_setbits( &can_send_flag, 1 ); 

	// reset both flags
	can_rx = CAN_NONE_IRQ;
	can_tx = CAN_NONE_IRQ;
}

// CAN Background send thread
void can_send_thread(cyg_addrword_t param)
{
	CANMsgStr can_str_msg;
	cyg_uint8 cnt=0;
	CANMsgType can_type_msg;

	diag_printf("Background CAN Send Thread Started!\n");
	// get the CAN data with the highest priority from the buffer and transmit it out forever
	while(1)
	{
//		diag_printf("Send Start Wait--semaphore!\n");
		// wait until the a semaphore is posted by user application
		cyg_semaphore_wait(&semaphore_send);
//		diag_printf("Send Pass Wait--semaphore!\n");

		node_get_hiprio_can_msg(&can_str_msg, MODE_SEND, MODE_REMOVE);

		can_type_msg.arbitration = translate_std_id(can_str_msg.std_id);
		can_type_msg.length = can_str_msg.length;
		for(cnt=0;cnt<can_type_msg.length;cnt++)
			can_type_msg.data[cnt] = can_str_msg.data[cnt];

		// then send it out
		SendStdCANMessage(can_type_msg);

//		diag_printf("Send Start Wait--flag!\n");
		// wait until the flag is set by the successfully transmission
		cyg_flag_wait( &can_send_flag, -1, CYG_FLAG_WAITMODE_OR | CYG_FLAG_WAITMODE_CLR );
//		diag_printf("Send Pass Wait--flag!\n");

		// do I need to signal the successfully transmission
		if(can_str_msg.can_block_mode == CAN_BLOCK)
		{
//			diag_printf("Check Block!\n");

//			diag_printf("Compare flag address: 0x%08x!\n", can_str_msg.pcan_block_flag);
			// set the flag bits
			cyg_flag_setbits( can_str_msg.pcan_block_flag, 1 ); 
			
//			diag_printf("Block Set Bits!\n");
		}
	}
}



// CAN Background receive thread
void can_recv_thread(cyg_addrword_t param)
{
	// can message buffer for receiving the can message
	CANMsgType can_type_msg;
	CANMsgStr can_str_msg;
	cyg_uint8 cnt=0;


	diag_printf("Background CAN Recv Thread Started!\n");
	while(1)
	{
		// wait until the flag get set by CAN receiver Interrupt
		cyg_flag_wait( &can_recv_flag, -1, CYG_FLAG_WAITMODE_OR | CYG_FLAG_WAITMODE_CLR );
		
		ReadCANMessage(&can_type_msg);

		can_str_msg.std_id = translate_std_id(can_type_msg.arbitration);
		can_str_msg.length = can_type_msg.length;
		for(cnt=0;cnt<can_str_msg.length;cnt++)
			can_str_msg.data[cnt] = can_type_msg.data[cnt];

		// need to push the received can message into the system buffer
		node_add_recv(can_str_msg);
	}
}

// this is low active reset
void reset_can_controller(void)
{
	cyg_uint32 i,j;
	volatile cyg_uint32 tmp;
	// make the port as output
	can_reset_pio->np_piodirection = 0xff;

	// reset the CAN controller, lower the resetN signal for a while
	can_reset_pio->np_piodata = 0x00;

	// delay for a while
	for(i=0;i<500;i++)
	{
		for(j=0;j<5000;j++)
			tmp = j;
	}

	// desert the reset signal
	can_reset_pio->np_piodata = 0x1;
}

void plddelay(cyg_uint32 cnt)
{
	volatile cyg_uint32 cnt1, cnt2, tmp;

	for(cnt1=0; cnt1<cnt; cnt1++)
	{
		for(cnt2=0;cnt2<50000;cnt2++)
		{
			tmp=cnt2;
		}
	}
}

// make the entry into the eCOS device table
// in .section ".ecos.table.devtab.begin"
// .global __DEVTAB__

// first define the required functions
// Function to initialize the device.  Called at bootstrap time.
bool soft_can_init(struct cyg_devtab_entry *tab)
{
    soft_can_channel *sc = (soft_can_channel *)tab->priv;

	// config the PLD data to setup soft CAN Controller implementation in PLD
	pld_config();

	plddelay(200);

    // Init the flag of waking up
	cyg_flag_init( &can_recv_flag);

    // Init the flag of successfully sending
	cyg_flag_init( &can_send_flag );

	// init mutex for sending
	cyg_mutex_init(&mutex_send);

	// init mutex for receiving
	cyg_mutex_init(&mutex_recv);

	// init the semaphore for sending to zero (continue when non zero)
	cyg_semaphore_init(&semaphore_send, 0);

	reset_can_controller();
	
	// initialise the soft can controller
	initHurryAMBA();

	// if using SOPC then we need to activate the PIO as Interrupt
	set_pio();

	// create interrupt service routine for soft can controller
	cyg_drv_interrupt_create(sc->interrupt,
                            0, // Priority - what goes here?
                             (cyg_addrword_t)sc, //  Data item passed to interrupt handler
                             (cyg_ISR_t *)soft_can_isr,
                             (cyg_DSR_t *)soft_can_dsr, // The logical driver DSR
                             &soft_can_interrupt_handle,
                             &soft_can_interrupt);
    cyg_drv_interrupt_attach(soft_can_interrupt_handle);
    cyg_drv_interrupt_acknowledge(sc->interrupt);
    cyg_drv_interrupt_unmask(sc->interrupt);


	// create a thread for receving from the Soft CAN Controller
	// Priortiy: 7
    cyg_thread_create(7,					// Priority 
                      can_recv_thread,			// entry
                      0,					// entry parameter
                      "CAN Recv Thread",			// Name
                      &can_recv_stack[0],		// Stack
                      4096,			// Size
                      &can_recv_thread_handle,	// Handle
                      &can_recv_thread_data		// Thread data structure
            );

	// create a thread for sending can data to the Soft CAN Controller  
	// Priority: 7
    cyg_thread_create(7,					// Priority 
                      can_send_thread,			// entry
                      0,					// entry parameter
                      "CAN Send Thread",			// Name
                      &can_send_stack[0],		// Stack
                      4096,			// Size
                      &can_send_thread_handle,	// Handle
                      &can_send_thread_data		// Thread data structure
            );


	// start the receive thread
    cyg_thread_resume(can_recv_thread_handle);

	// start the send thread
    cyg_thread_resume(can_send_thread_handle);


	return ENOERR;
}

// This routine is called when the device is "looked" up (i.e. attached)
Cyg_ErrNo soft_can_lookup(struct cyg_devtab_entry **tab, struct cyg_devtab_entry *sub_tab, const char *name)
{
	return ENOERR;
}

// make the block mode device table entry into the eCOS device driver table
BLOCK_DEVTAB_ENTRY(soft_can_table_entry, 
				   can_controller_name, 
				   0, 
				   &soft_can_io_handler_table, 
				   soft_can_init, 
				   NULL, 
				   &soft_can_channel0)
