#include <cyg/io/can_lib.h>

unsigned int get_can_ctrl(void)
{

	volatile unsigned int *H = (unsigned int *)HA_BASE;
	unsigned int tmp = H[0];

	return tmp;

}

unsigned int get_can_status(void)
{

	volatile unsigned int *H = (unsigned int *)HA_BASE;
	unsigned int tmp = H[1];

	return tmp;

}

unsigned int SingleBit(unsigned char i) 
{
	unsigned int b = 1;
	if (i<32) 
	{
		return b << i;
	}
	
	else
		return b << 31;
}

void ReqSend(void)
{
	volatile unsigned int *H = (unsigned int *)HA_BASE;
	*H = *H | 0x08000000;
}

void SetSetupBit(SETUP_BIT bit) 
{
	volatile unsigned int *H = (unsigned int *)HA_BASE;
	*H = *H | SingleBit(bit);
}

void ClearSetupBit(SETUP_BIT bit) 
{
	volatile unsigned int *H = (unsigned int *)HA_BASE;
	*H = *H & ~SingleBit(bit);
}

unsigned char StatusBit(STATUS_BIT bit) 
{
	volatile unsigned int *H = (unsigned int *)(HA_BASE+4);

	if (*H & SingleBit(bit)) 
		return 1;
	else 
		return 0;
}

void WriteTXCanMessage(CANMsgType can_type_msg) 
{
	volatile unsigned int *H = (unsigned int *)HA_BASE;
	H[3] = can_type_msg.arbitration;
	H[4] = (((unsigned int)can_type_msg.data[3])<<24)|
	(((unsigned int)can_type_msg.data[2])<<16)|
	(((unsigned int)can_type_msg.data[1])<<8)|
	((unsigned int)can_type_msg.data[0]);
	H[5] = (((unsigned int)can_type_msg.data[7])<<24)|
	(((unsigned int)can_type_msg.data[6])<<16)|
	(((unsigned int)can_type_msg.data[5])<<8)|
	((unsigned int)can_type_msg.data[4]);
}

void SetTXMsgLength(unsigned char l)
{
	volatile unsigned int *H = (unsigned int *)HA_BASE;
	if (l < 9) 
	{
		H[0] = H[0] & 0xfffff0ff;		
		if (l & 1) 
			H[0] = H[0] | SingleBit(TXDLC0);
		if (l & 2) 
			H[0] = H[0] | SingleBit(TXDLC1);
		if (l & 4) 
			H[0] = H[0] | SingleBit(TXDLC2);
		if (l & 8) 
			H[0] = H[0] | SingleBit(TXDLC3);

//		diag_printf("In Set Length, l=0x%02x, Ctrl: 0x%04x, Status: 0x%04x\n", l, get_can_ctrl(), get_can_status() );
	}
}

void WriteFilter(FilterType fil) 
{
	volatile unsigned int *H = (unsigned int *)HA_BASE;

//	diag_printf("Set Mask Filter! mask: 0x%04x, cod: 0x%04x\n", fil.mask, fil.code);

	H[2] = (((unsigned int)fil.mask)&0x0000FFFF) | ((unsigned int)fil.code)<<16;

//	diag_printf("Mask Filter: 0x%08x\n", H[2]);
}

FilterType Filter(void) 
{
	volatile unsigned int *H = (unsigned int *)HA_BASE;
	unsigned int tmp;
	FilterType fil;
	tmp = H[2];
	fil.mask = (unsigned short)(tmp&0x0000FFFF);
	fil.code = (unsigned short)(tmp>>16);
	return fil;
}

CANMsgType RXMessage(void)
{
	volatile unsigned int *H = (unsigned int *)HA_BASE;
	unsigned int tmp;
	CANMsgType can_type_msg;
	can_type_msg.arbitration = H[6];
	tmp = H[7];
	can_type_msg.data[3] = (unsigned char)((tmp >> 24)&0xff);
	can_type_msg.data[2] = (unsigned char)((tmp >> 16)&0xff);
	can_type_msg.data[1] = (unsigned char)((tmp >> 8)&0xff);
	can_type_msg.data[0] = (unsigned char)((tmp)&0xff);
	tmp = H[8];
	can_type_msg.data[7] = (unsigned char)((tmp >> 24)&0xff);
	can_type_msg.data[6] = (unsigned char)((tmp >> 16)&0xff);
	can_type_msg.data[5] = (unsigned char)((tmp >> 8)&0xff);
	can_type_msg.data[4] = (unsigned char)((tmp)&0xff);
	return can_type_msg;
}

unsigned char RXMsgLength(void)
{
	volatile unsigned int *H = (unsigned int *)HA_BASE;
	unsigned int tmp = H[1];


//	diag_printf("CAN Status: 0x%04x\n", tmp);


	return	((unsigned char)((tmp&SingleBit(RXDLC0))?1:0)) |
			((unsigned char)((tmp&SingleBit(RXDLC1))?2:0)) |
			((unsigned char)((tmp&SingleBit(RXDLC2))?4:0)) |
			((unsigned char)((tmp&SingleBit(RXDLC3))?8:0));
}

unsigned char RXErrCnt(void) 
{
	volatile unsigned int *H = (unsigned int *)HA_BASE;
	return (unsigned char )H[9];
}

unsigned char TXErrCnt(void) 
{
	volatile unsigned int *H = (unsigned int *)HA_BASE;
	return (unsigned char )(H[9]>>8);
}

void WriteTriggerMatch(TriggerMatchType t) 
{
	volatile unsigned int *H = (unsigned int *)HA_BASE;
	H[10] = (unsigned int)t;
}

TriggerMatchType TriggerMatch(void) 
{
	volatile unsigned int *H = (unsigned int *)HA_BASE;
	return (TriggerMatchType )H[10];
}

void SendStdCANMessage(CANMsgType can_type_msg) 
{
	volatile unsigned int *H = (unsigned int *)HA_BASE;

	volatile cyg_uint32 temp;

	WriteTXCanMessage(can_type_msg);

	SetTXMsgLength(can_type_msg.length);

	// clear extended frame flag 
	ClearSetupBit(TXEM);

	// This bit establishes if the transmitted message is a remote frame
	ClearSetupBit(TXRM);

	// This bit establishes if the frame has the remote frame bit set, such as to request a remote frame.
	ClearSetupBit(TMRMR);

	// read back all the setting from the soft can controller

	temp = H[3];	// can_type_msg.arbitration;
	diag_printf("CAN ID: 0x%02x, Length: 0x%02x\n", translate_std_id(temp), can_type_msg.length);


	temp = H[4];	// (((unsigned int)can_type_msg.data[3])<<24)| (((unsigned int)can_type_msg.data[2])<<16)|(((unsigned int)can_type_msg.data[1])<<8)|((unsigned int)can_type_msg.data[0]);
	diag_printf("Data: 0x%02x, 0x%02x, 0x%02x, 0x%02x, ", (temp&0xff), ((temp&0xff00)>>8), ((temp&0xff0000)>>16), ((temp&0xff000000)>>24) );

	temp = H[5];	// (((unsigned int)can_type_msg.data[7])<<24)|(((unsigned int)can_type_msg.data[6])<<16)|(((unsigned int)can_type_msg.data[5])<<8)|((unsigned int)can_type_msg.data[4]);
	diag_printf("0x%02x, 0x%02x, 0x%02x, 0x%02x\n", (temp&0xff), ((temp&0xff00)>>8), ((temp&0xff0000)>>16), ((temp&0xff000000)>>24) );

	SetSetupBit(TXReq);

	diag_printf("CAN Error Counter: 0x%08x\n", H[9]);

}

void ReadCANMessage(CANMsgType *can_type_msg) 
{
	int i;
	while(!StatusBit(RXOk));
	
	*can_type_msg = RXMessage();
	can_type_msg->length = RXMsgLength();

//	diag_printf("Check Received Message Length: 0x%02x\n", can_type_msg->length);
	
	diag_printf("Received CAN Message:!\n");
	diag_printf("STD ID: 0x%02x, Length: 0x%02x\n",	translate_std_id(can_type_msg->arbitration), can_type_msg->length);
	diag_printf("Data: 0x%02x,  0x%02x,  0x%02x,  0x%02x,  0x%02x,  0x%02x,  0x%02x,  0x%02x\n", 
	can_type_msg->data[0],
	can_type_msg->data[1],
	can_type_msg->data[2],
	can_type_msg->data[3], 
	can_type_msg->data[4],
	can_type_msg->data[5],
	can_type_msg->data[6],
	can_type_msg->data[7]
	);

	// clear the RX flag
	SetSetupBit(RXClr);
}

// Arbitration format <--> Standard CAN ID format
cyg_uint16 translate_std_id(cyg_uint16 std_id)
{
	cyg_uint8 cnt;
	cyg_uint16 temp_id;

	temp_id=0;
	for(cnt=0;cnt<11;cnt++)
	{
		if( (1<<cnt) & std_id )
			temp_id |= (1<< (10-cnt));		
	}
	return temp_id;
}

// Arbitration format <--> Standard CAN ID format
cyg_uint16 translate_ext_id(cyg_uint16 ext_id)
{
	cyg_uint8 cnt=0;
	cyg_uint16 temp_id=0;

	// not implemented here yet

	return temp_id;

}

// set CAN speed at 1Mpbs with 16MHz clock input
void setup1M(void)
{
	// set 1Mbps
	SetSetupBit(Disabled);

	// setup the speed for 500Kbits/s

	// PS1_0 ~ PS1_3 = 0b1000 : 8
	SetSetupBit(PS1_3);
	ClearSetupBit(PS1_2);
	ClearSetupBit(PS1_1);
	ClearSetupBit(PS1_0);

	// PS2_0 ~ PS2_3 = 0b0111 : 7
	ClearSetupBit(PS2_3);
	SetSetupBit(PS2_2);
	SetSetupBit(PS2_1);
	SetSetupBit(PS2_0);
	
	// RSJ0 ~ RSJ2 -> 0b010
	// SJW = 2;
	ClearSetupBit(RSJ2);
	SetSetupBit(RSJ1);
	ClearSetupBit(RSJ0);

	// Setup BRP: 0b00 -> Pclock --> 16MHz
	ClearSetupBit(BPR1);
	ClearSetupBit(BPR0);	

	ClearSetupBit(Disabled);
}

// set the CAN speed to 500Kbps with 16MHz clock input
void setup500k(void)
{
	// set 500bps
	SetSetupBit(Disabled);

	// setup the speed for 500Kbits/s

	// PS1_0 ~ PS1_3 = 0b1000 : 8
	SetSetupBit(PS1_3);
	ClearSetupBit(PS1_2);
	ClearSetupBit(PS1_1);
	ClearSetupBit(PS1_0);

	// PS2_0 ~ PS2_3 = 0b0111 : 7
	ClearSetupBit(PS2_3);
	SetSetupBit(PS2_2);
	SetSetupBit(PS2_1);
	SetSetupBit(PS2_0);
	
	// RSJ0 ~ RSJ2 -> 0b010
	// SJW = 2;
	ClearSetupBit(RSJ2);
	SetSetupBit(RSJ1);
	ClearSetupBit(RSJ0);

	// Setup BRP: 0b01 -> Pclock/2 --> 8MHz
	ClearSetupBit(BPR1);
	SetSetupBit(BPR0);	

	ClearSetupBit(Disabled);
}

// set the CAN speed to 250Kbps with 16MHz clock input
void setup250k(void)
{
	// set 250bps
	SetSetupBit(Disabled);

	// setup the speed for 250Kbits/s

	// PS1_0 ~ PS1_3 = 0b0100 : 4
	ClearSetupBit(PS1_3);
	SetSetupBit(PS1_2);
	ClearSetupBit(PS1_1);
	ClearSetupBit(PS1_0);

	// PS2_0 ~ PS2_3 = 0b0011 : 3
	ClearSetupBit(PS2_3);
	ClearSetupBit(PS2_2);
	SetSetupBit(PS2_1);
	SetSetupBit(PS2_0);
	
	// RSJ0 ~ RSJ2 -> 0b010
	// SJW = 2;
	ClearSetupBit(RSJ2);
	SetSetupBit(RSJ1);
	ClearSetupBit(RSJ0);

	// Setup BRP: 0b10 -> Pclock/8 --> 2MHz
	SetSetupBit(BPR1);
	ClearSetupBit(BPR0);	

	ClearSetupBit(Disabled);
}

// set the CAN speed to 125Kbps with 16MHz clock input
void setup125k(void)
{
	// set 125bps
	SetSetupBit(Disabled);

	// setup the speed for 125Kbits/s

	// PS1_0 ~ PS1_3 = 0b1000 : 8
	SetSetupBit(PS1_3);
	ClearSetupBit(PS1_2);
	ClearSetupBit(PS1_1);
	ClearSetupBit(PS1_0);

	// PS2_0 ~ PS2_3 = 0b0111 : 7
	ClearSetupBit(PS2_3);
	SetSetupBit(PS2_2);
	SetSetupBit(PS2_1);
	SetSetupBit(PS2_0);
	
	// RSJ0 ~ RSJ2 -> 0b010
	// SJW = 2;
	ClearSetupBit(RSJ2);
	SetSetupBit(RSJ1);
	ClearSetupBit(RSJ0);

	// Setup BRP: 0b10 -> Pclock/8 --> 2MHz
	SetSetupBit(BPR1);
	ClearSetupBit(BPR0);	

	ClearSetupBit(Disabled);
}

// set the CAN speed to 50Kbps with 16MHz clock input
void setup50k(void)
{
	// set 50bps
	SetSetupBit(Disabled);

	// setup the speed for 50Kbits/s

	// PS1_0 ~ PS1_3 = 0b0101 : 5
	ClearSetupBit(PS1_3);
	SetSetupBit(PS1_2);
	ClearSetupBit(PS1_1);
	SetSetupBit(PS1_0);

	// PS2_0 ~ PS2_3 = 0b0100 : 4
	ClearSetupBit(PS2_3);
	SetSetupBit(PS2_2);
	ClearSetupBit(PS2_1);
	ClearSetupBit(PS2_0);
	
	// RSJ0 ~ RSJ2 -> 0b010
	// SJW = 2;
	ClearSetupBit(RSJ2);
	SetSetupBit(RSJ1);
	ClearSetupBit(RSJ0);

	// Setup BRP: 0b11 -> Pclock/32 --> 500KHz
	SetSetupBit(BPR1);
	SetSetupBit(BPR0);	

	ClearSetupBit(Disabled);
}

// config the can controller speed
cyg_uint8 CAN_set_speed(cyg_uint8 speed)
{
	cyg_uint8 flag = CAN_NOERROR;

	// 25MHz input clock
	switch(speed)
	{
		case CAN_1M:
//			diag_printf("Speed@ 1Mbps\n");
			setup1M();
			break;

		case CAN_500K:
//			diag_printf("Speed@ 500Kbps\n");
			setup500k();
			break;

		case CAN_250K:
//			diag_printf("Speed@ 250Kbps\n");
			setup250k();
			break;

		case CAN_125K:
//			diag_printf("Speed@ 125Kbps\n");
			setup125k();
			break;

		case CAN_50K:
//			diag_printf("Speed@ 50Kbps\n");
			setup50k();
			break;

		default:
			// other speed requirment which has not been implemented then returns the error flag
			flag = CAN_ERROR;
			break;
	}

	return flag;
}

// config the can controller speed
cyg_uint8 CAN_set_mask_filter(FilterType mask_filter)
{
	volatile unsigned int *H = (unsigned int *)HA_BASE;

	// output the mask and the filter to be written to the CAN controller
	diag_printf("CAN Mask: 0x%04x, Filter: 0x%04x\n", mask_filter.mask, mask_filter.code);

	// first disable the CAN controller
	SetSetupBit(Disabled);

	// write the filter and mask to the CAN controller
	WriteFilter(mask_filter);

	// enable the CAN controller again
	ClearSetupBit(Disabled);

	return CAN_NOERROR;
}


void initHurryAMBA(void) 
{
	// reset the CAN controller
	SetSetupBit(Reset);

	// disable the CAN controller before setting the register
	SetSetupBit(Disabled);

	// set the CAN controller speed to 500Kbps
	CAN_set_speed(CAN_500K);

	// clear the RX 
	SetSetupBit(RXClr);

	// clear the IRQ
	SetSetupBit(IntClr);

	// TX Interrupt Enabled
	SetSetupBit(TXInt);

	// RX IRQ enabled
	SetSetupBit(RXInt);

	// enable error condition interrupts
	SetSetupBit(OverInt);
	SetSetupBit(ErrInt);

	// enable CAN controller
	ClearSetupBit(Disabled);

}
