#include <cyg/io/can_node.hxx>

// declare the can msg list class object
static Cyg_CANMSG_LIST m_recv_canmsg_list;
static Cyg_CANMSG_LIST m_send_canmsg_list;

static Cyg_RECV_FLAG_LIST m_recv_flag_list;

cyg_mutex_t mutex_recv_list;

// Cyg_CANMSG_LIST class member 
Cyg_CANMSG_NODE* Cyg_CANMSG_LIST::get_highest_can_msg_node(void)
{
	Cyg_CANMSG_NODE *pnext_node, *pnode;
	
	// initialise the pnode points to the head of this list
	pnode = get_head();

	// if no can msg node in this list the return with message printed out
	if(pnode == NULL)
	{
//		diag_printf("NULL LIST!\n");
		return NULL;
	}

	// the the next can msg node to the head of this list
	pnext_node = pnode->get_next();

	// travel through the list to find the can msg node with the highest priority
	while( (pnext_node != get_head()) && (pnext_node != NULL) )
	{
		// if the next can msg node has the higher priority then 
		if( pnode->std_id > pnext_node->std_id)
		{	
			pnode = pnext_node;
		}
		
		// advance the pnext_node to the next
		pnext_node = pnext_node->get_next();
	}


	// by now the *pnode points to the can msg node with the highest priority
	
	return pnode;
}

// get the CAN message node from the list with the specified CAN ID
Cyg_CANMSG_NODE* Cyg_CANMSG_LIST::get_can_msg_node(cyg_uint16 std_id, cyg_uint16 ext_id)
{
	Cyg_CANMSG_NODE *pcur_node, *pnode;
	
	pnode = NULL;

	// initialise the pnode points to the head of this list
	pcur_node = get_head();

	do
	{
		if(pcur_node != NULL)
		{
			if(pcur_node->std_id == std_id)
			{
				pnode = pcur_node;
				break;
			}

			else
				pcur_node = pcur_node->get_next();
		}
		
	} while( (pcur_node != get_head()) && (pcur_node != NULL) );

	return pnode;
}


void check_node(Cyg_CANMSG_NODE *pnode)
{
	Cyg_RECV_FLAG_NODE *pcur_node;

	cyg_mutex_lock(&mutex_recv_list);

	pcur_node = m_recv_flag_list.get_head();

	do
	{
		if(pcur_node != NULL)
		{
			if(pcur_node->std_id == pnode->std_id)
			{
				// signal the flag
				cyg_flag_setbits( pcur_node->pcan_block_flag, 1 ); 

//				diag_printf("Recv List Flag Set!\n");
				break;
			}

			else
				pcur_node = pcur_node->get_next();
		}
		
	} while( (pcur_node != m_recv_flag_list.get_head()) && (pcur_node != NULL) );


	cyg_mutex_unlock(&mutex_recv_list);

}


// a can msg received then initialise a Cyg_CANMSG node and add to list
extern "C" cyg_uint8 node_add_recv(CANMsgStr can_str_msg)
{
    Cyg_CANMSG_NODE *pnode;
	cyg_uint8 cnt=0;

	// create a message buffer and return the pointer
	pnode = new Cyg_CANMSG_NODE;

	diag_printf("Create new node for recv list at address: 0x%08x\n", pnode);

	if(pnode==NULL)
	{
//		diag_printf("Memory error!\n");
		return CAN_ERROR;
	}

	// initialise the newly allocated can message buffer
	pnode->std_id = can_str_msg.std_id;
	pnode->ext_id = can_str_msg.ext_id;
	pnode->length = can_str_msg.length;
	for(cnt=0;cnt<can_str_msg.length;cnt++)
		pnode->data[cnt] = can_str_msg.data[cnt];
	pnode->can_block_mode = can_str_msg.can_block_mode;
	pnode->pcan_block_flag = can_str_msg.pcan_block_flag;


	// add the node to the can recv list
	m_recv_canmsg_list.add_tail(pnode);

//	diag_printf("Added Received Node!\n");
//	diag_printf("STD ID: 0x%02x, Length: 0x%02x, Data: 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x\n",
//		pnode->std_id,
//		pnode->length,
//		pnode->data[0],
//		pnode->data[1],
//		pnode->data[2],
//		pnode->data[3],
//		pnode->data[4],
//		pnode->data[5],
//		pnode->data[6],
//		pnode->data[7]);


	check_node(pnode);

	return CAN_NOERROR;
}

// add the send message to the buffer list
extern "C" cyg_uint8 node_add_send(CANMsgStr can_str_msg)
{
    Cyg_CANMSG_NODE *pnode;
	cyg_uint8 cnt=0;

	// create a message buffer and return the pointer
	pnode = new Cyg_CANMSG_NODE;
	
	diag_printf("Create new node for send list at address: 0x%08x\n", pnode);

	if(pnode==NULL)
	{
//		diag_printf("Memory error!\n");
		return CAN_ERROR;
	}

	// initialise the newly allocated can message buffer
	pnode->std_id = can_str_msg.std_id;
	pnode->ext_id = can_str_msg.ext_id;
	pnode->length = can_str_msg.length;

	for(cnt=0;cnt<8;cnt++)
		pnode->data[cnt] = can_str_msg.data[cnt];

	pnode->can_block_mode = can_str_msg.can_block_mode;
	pnode->pcan_block_flag = can_str_msg.pcan_block_flag;

//	diag_printf("node add send PASSED flag address: 0x%08x!\n", can_str_msg.pcan_block_flag);
//	diag_printf("node add send flag address: 0x%08x!\n", pnode->pcan_block_flag);

	// add the node to the can recv list
	m_send_canmsg_list.add_tail(pnode);

	return CAN_NOERROR;
}

// function to get the highest priority node info
// op_mode1 : to deal with send buffer list (MODE_SEND) or receive buffer list (MODE_RECV)
// op_mode2 : whether the node to be removed or not. (MODE_REMOVE ,MODE_NOREMOVE)
extern "C" cyg_uint8 node_get_hiprio_can_msg(CANMsgStr *pcan_str_msg, cyg_uint8 op_mode1, cyg_uint8 op_mode2)
{
	Cyg_CANMSG_NODE *pnode;
	cyg_uint8 cnt=0;
	cyg_uint8 flag;

	// if ask for the send buffer list
	if(op_mode1 == MODE_SEND)
	{		
		// retrieve the can message with the highest priority then if removing required then remove it from the list and free the memory
		pnode = m_send_canmsg_list.get_highest_can_msg_node();

		if(pnode == NULL)
			return CAN_ERROR;

		// now the can node with the highest priority is pointed by *pnode
		// passe the retrieved can message back to caller
		pcan_str_msg->std_id = pnode->std_id;
//		can_type_msg->arbitration = translate_ext_id(pnode->ext_id);
		pcan_str_msg->length = pnode->length;
		for(cnt=0; cnt<pcan_str_msg->length; cnt++)
			pcan_str_msg->data[cnt] = pnode->data[cnt];
		pcan_str_msg->can_block_mode = pnode->can_block_mode;
	
		pcan_str_msg->pcan_block_flag = pnode->pcan_block_flag;

//		diag_printf("get hi flag address: 0x%08x!\n", pcan_str_msg->pcan_block_flag);


		// remove the node from the list if needed
		if(op_mode2 == MODE_REMOVE)
		{
			m_send_canmsg_list.remove(pnode);

			diag_printf("Delete Node at address: 0x%08x\n");
			// and free the memory resource
			delete pnode;
		}
	}

	// if ask for the receive buffer list
	else if(op_mode1 == MODE_RECV)
	{
		// retrieve the can message with the highest priority then if removing required then remove it from the list and free the memory
		pnode = m_recv_canmsg_list.get_highest_can_msg_node();

		if(pnode == NULL)
			return CAN_ERROR;

		// now the can node with the highest priority is pointed by *pnode
		// passe the retrieved can message back to caller
		pcan_str_msg->std_id = pnode->std_id;
//		can_type_msg->arbitration = translate_ext_id(pnode->ext_id);
		pcan_str_msg->length = pnode->length;
		for(cnt=0; cnt<pcan_str_msg->length; cnt++)
			pcan_str_msg->data[cnt] = pnode->data[cnt];
		pcan_str_msg->can_block_mode = pnode->can_block_mode;
		pcan_str_msg->pcan_block_flag = pnode->pcan_block_flag;


		// remove the node from the list
		if(op_mode2 == MODE_REMOVE)
		{
			m_recv_canmsg_list.remove(pnode);

			diag_printf("Delete node at address: 0x%08x\n", pnode);

			// and free the memory resource
			delete pnode;
		}
	}

	return CAN_NOERROR;
}

// get a node from head from a can msg buffer list, then remove it
extern "C" cyg_uint8 node_get_can_head(CANMsgStr *pcan_str_msg, cyg_uint8 mode)
{	
	Cyg_CANMSG_NODE *pnode;
	cyg_uint8 flag = CAN_ERROR;

	if(mode == MODE_RECV)
	{
		pnode = m_recv_canmsg_list.get_head();
		pcan_str_msg->std_id = m_recv_canmsg_list.get_head()->std_id;
//		can_type_msg->arbitration = translate_ext_id(m_recv_canmsg_list.get_head()->ext_id);
		pcan_str_msg->length = m_recv_canmsg_list.get_head()->length;
		pcan_str_msg->data[0] = m_recv_canmsg_list.get_head()->data[0];
		pcan_str_msg->data[1] = m_recv_canmsg_list.get_head()->data[1];
		pcan_str_msg->data[2] = m_recv_canmsg_list.get_head()->data[2];
		pcan_str_msg->data[3] = m_recv_canmsg_list.get_head()->data[3];
		pcan_str_msg->data[4] = m_recv_canmsg_list.get_head()->data[4];
		pcan_str_msg->data[5] = m_recv_canmsg_list.get_head()->data[5];
		pcan_str_msg->data[6] = m_recv_canmsg_list.get_head()->data[6];
		pcan_str_msg->data[7] = m_recv_canmsg_list.get_head()->data[7];
		pcan_str_msg->can_block_mode = pnode->can_block_mode;
		pcan_str_msg->pcan_block_flag = pnode->pcan_block_flag;

		m_recv_canmsg_list.rem_head();	

		diag_printf("Delete node at address: 0x%08x\n", pnode);

		// free the memory resource
		delete pnode;

		flag = CAN_NOERROR;
	}

	// look into the send buffer list
	else if(mode == MODE_SEND)
	{
		pnode = m_send_canmsg_list.get_head();
		pcan_str_msg->std_id = translate_std_id(m_send_canmsg_list.get_head()->std_id);
//		can_type_msg->arbitration = translate_ext_id(m_send_canmsg_list.get_head()->ext_id);
		pcan_str_msg->length = m_send_canmsg_list.get_head()->length;
		pcan_str_msg->data[0] = m_send_canmsg_list.get_head()->data[0];
		pcan_str_msg->data[1] = m_send_canmsg_list.get_head()->data[1];
		pcan_str_msg->data[2] = m_send_canmsg_list.get_head()->data[2];
		pcan_str_msg->data[3] = m_send_canmsg_list.get_head()->data[3];
		pcan_str_msg->data[4] = m_send_canmsg_list.get_head()->data[4];
		pcan_str_msg->data[5] = m_send_canmsg_list.get_head()->data[5];
		pcan_str_msg->data[6] = m_send_canmsg_list.get_head()->data[6];
		pcan_str_msg->data[7] = m_send_canmsg_list.get_head()->data[7];
		pcan_str_msg->can_block_mode = pnode->can_block_mode;
		pcan_str_msg->pcan_block_flag = pnode->pcan_block_flag;

		m_send_canmsg_list.rem_head();

		diag_printf("Delete node at address: 0x%08x\n", pnode);

		// free the memory resource
		delete pnode;

		flag = CAN_NOERROR;
	}

	return flag;
}

// get the specified CAN message from the list
// remove it from the buffer is needed
extern "C" cyg_uint8 node_get_msg(CANMsgStr *pcan_str_msg, cyg_uint8 mode)
{
	Cyg_CANMSG_NODE *pnode;
	cyg_uint8 cnt, flag=CAN_ERROR;

	pnode = m_recv_canmsg_list.get_can_msg_node(pcan_str_msg->std_id, pcan_str_msg->ext_id);

	// if the node pointer returned is valid
	if(pnode != NULL)
	{
		// copy the can data
		pcan_str_msg->length = pnode->length;
		
		for(cnt=0;cnt<pcan_str_msg->length; cnt++)
			pcan_str_msg->data[cnt] = pnode->data[cnt];

		pcan_str_msg->can_block_mode = pnode->can_block_mode;
//		pcan_str_msg->pcan_block_flag = pnode->pcan_block_flag;

//		diag_printf("CAN Msg from List!\n");
//		diag_printf("STD ID: 0x%02x, Length: 0x%02x, Data: 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x\n",
//			pcan_str_msg->std_id,
//			pcan_str_msg->length,
//			pcan_str_msg->data[0],
//			pcan_str_msg->data[1],
//			pcan_str_msg->data[2],
//			pcan_str_msg->data[3],
//			pcan_str_msg->data[4],
//			pcan_str_msg->data[5],
//			pcan_str_msg->data[6],
//			pcan_str_msg->data[7]);


		// if removal needed then do it
		if(mode == MODE_REMOVE)
		{
			m_recv_canmsg_list.remove(pnode);

			diag_printf("Delete Node at address: 0x%08x\n", pnode);

			// free the memory
			delete pnode;
		}
		
		// set the flag to indicate the message retrieved
		flag = CAN_NOERROR;
	}

	else
	{
//		diag_printf("NULL List Returned!\n");
		flag = CAN_ERROR;
	}

	return flag;	
}

extern "C" cyg_uint8 node_get_msg_block(CANMsgStr *pcan_str_msg, cyg_uint8 mode)
{
	// init flag and
	cyg_flag_t	flag_recv_list;
	Cyg_RECV_FLAG_NODE *pnode;

	// flag list object
	pnode = new Cyg_RECV_FLAG_NODE;

	cyg_flag_init(&flag_recv_list);

	cyg_mutex_init(&mutex_recv_list);

	pcan_str_msg->pcan_block_flag = &flag_recv_list;
	
	// add the flag and id to list
	pnode->std_id = pcan_str_msg->std_id;
	pnode->ext_id = pcan_str_msg->ext_id;
	pnode->pcan_block_flag = pcan_str_msg->pcan_block_flag;

	cyg_mutex_lock(&mutex_recv_list);

	m_recv_flag_list.add_tail(pnode);

	cyg_mutex_unlock(&mutex_recv_list);

	cyg_flag_wait( &flag_recv_list, -1, CYG_FLAG_WAITMODE_OR | CYG_FLAG_WAITMODE_CLR );
	
	node_get_msg(pcan_str_msg, mode);

	// once the message has been received then free the memory
	diag_printf("Delete Node at address: 0x%08x\n", pnode);
	delete pnode;
}

/*
hqclass m_hqclass;

extern "C" hqclass* get_hqclass(void)
{
	return &m_hqclass;
}

extern "C" void call_hqfun1(hqclass* m_obj)
{

	m_obj->hqfun1();
}

extern "C" void call_hqfun2(hqclass* m_obj)
{

	m_obj->hqfun2();
}

extern "C" void set_ab(hqclass* m_obj, unsigned char m_a, unsigned char m_b)
{
	m_obj->set_a_b(m_a, m_b);
}
*/


