#include <cyg/io/pld_config.h>

// read a word at pos from filedata[] array, ** start from word 0  **
cyg_uint32 Read_File_Word(unsigned char confdata[], unsigned long pos)
{
	return ( confdata[pos*4] | (confdata[pos*4+1]<<8) | (confdata[pos*4+2]<<16) | (confdata[pos*4+3]<<24) );
//	return ( (filedata[pos*4]<<24) | (filedata[pos*4+1]<<16) | (filedata[pos*4+2]<<8) | filedata[pos*4+3] );

}

unsigned char pld_config(void)
{
	cyg_uint32 read_val, pld_data, size, start_pos, end_pos, cnt;
	
	unsigned long len=130000;
	unsigned char* unpacked = malloc(len);

	// display message
	diag_printf("Start Configuring PLD Device!\n");


	// first should decompress the PLD configuration data into array
	// doing decompression and output result
    if (NULL == unpacked)
		diag_printf("Not enough memory for buffers\n");
	else
		uncompress(unpacked, &len, filedata, sizeof(filedata));

	// set the config_clock period to 7.8125MHz
	HAL_WRITE_UINT32(EPXA1_CONFIG_CLOCK, 0x04);

	// check the lock bit of config_control
	HAL_READ_UINT32(EPXA1_CONFIG_CRTL, read_val);
	
	// if PLD locked then return error
	if(read_val == 1)
	{
		diag_printf("PLD Locked! Error! Return!\n");
		return PLD_ERROR;
	}

	// set co bit of config_control
	read_val |= 0x2;
	HAL_WRITE_UINT32(EPXA1_CONFIG_CRTL, read_val);

	// get the ID of the chip
	HAL_READ_UINT32(EPXA1_IDCODE, read_val);
	// get ID from the configuration file
	pld_data = Read_File_Word(unpacked, 1);

	// if both IDs don't match then return error
	if(pld_data != read_val)
	{
		diag_printf("ID Code from file: 0x%08x\n", pld_data);
		diag_printf("ID Code from Device: 0x%08x\n", read_val);

		diag_printf("Chip ID not match! Error! Return!\n");
		return PLD_ERROR;
	}

	// Start address in: word 2, size(in bytes) in word 3 ( word 0, word 1, word 2, word 3)
	start_pos = (Read_File_Word(unpacked, 2)/4);
	size = Read_File_Word(unpacked, 3);
	size /= 4; // change into word size
	end_pos = start_pos + size;

	diag_printf("Start Pos: 0x%08x, Size(in words): 0x%08x\n", start_pos, size);

	// start configuring
	for(cnt=start_pos; cnt<end_pos; cnt++)
	{
		// get the configuration data from file and write to register: "EPXA1_CONFIG_DATA"
		pld_data = Read_File_Word(unpacked, cnt);
		HAL_WRITE_UINT32(EPXA1_CONFIG_DATA, pld_data);

#if PrintDEBUG 
		diag_printf("PLD Config Data: 0x%08x\n", pld_data);
#endif

		// test the busy bit from EPXA1_CONFIG_CRTL
		pld_data = 0xff;
		while(pld_data & 0x04)
		{
			HAL_READ_UINT32(EPXA1_CONFIG_CRTL, pld_data);

#if PrintDEBUG 
			diag_printf("PLD Config Status: 0x%08x\n", pld_data);
#endif
		}
	}

	
	// check for CO bit (wait until whole configuration finished)
	pld_data = 0xff;
	while(pld_data & 0x02)
	{
		HAL_READ_UINT32(EPXA1_CONFIG_CRTL, pld_data);

#if PrintDEBUG & 5
			diag_printf("PLD Whole Config Status: 0x%08x\n", pld_data);
#endif

	}

	// check error flag bit in 
	HAL_READ_UINT32(EPXA1_CONFIG_CRTL, pld_data);
	if(pld_data & 0x10)
	{
		diag_printf("PLD Config Error! Return!\n");
		return PLD_ERROR;
	}

	diag_printf("PLD Configuration Done Successfully!\n");

	return PLD_CONOK;
}
