//==========================================================================
//
//      gate_way.c
//
//      Ethernet <--> CAN bus gate way system
//
//==========================================================================
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    Qiang Huang
// Contributors: None
// Date:         2003-05-13
// Purpose:      Demonstration
// Description:  
//              
//
//####DESCRIPTIONEND####
//
//==========================================================================

// include the necessary header for networking
#include <network.h>

// CAN interface
#include <cyg/io/can_api.h>

#include <pkgconf/net.h>
#include <pkgconf/httpd.h>
#include <cyg/httpd/httpd.h>



// flash memory file system
#include <pkgconf/fs_jffs2.h>	// Address of JFFS2
#include <pkgconf/io_fileio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <string.h>
#include <dirent.h>
#include <cyg/fileio/fileio.h>
#include <cyg/io/flash.h>
#define IOSIZE  100



static cyg_bool cyg_web_server( FILE * client, char *filename,
                                    char *formdata, void *arg )
{
    char *formlist[100];
    cyg_uint32 base = 0;
    unsigned int datasize = 1;
    int size = 256;
    char *p;
	cyg_uint32 std_id=0;
	cyg_uint32 can_data[8]={0};
	cyg_uint32 dummycan_data[8]={0};
	cyg_uint8 dummydata[8]={0};
	cyg_uint32 can_len=0, cnt=0;
	CANMsgStr can_str_msg;
	cyg_uint8 dirt=0;

	cyg_uint32 rstd_id=0;
	cyg_uint32 rcan_data[8]={0};
	cyg_uint32 rcan_len=0, rcnt=0;
	CANMsgStr rcan_str_msg;
	cyg_uint8 rdirt=0;

	cyg_uint8 flash_data[30]="Flash Data Here";
	cyg_uint8 fsfile_name[30]="Flash File Name Here";

	cyg_uint8 ram_data[30]="RAM Data Here";
	cyg_uint8 ramfile_name[30]="RAM File Name Here";

	cyg_uint32 speed=0, mask=0, filter=0;


    cyg_formdata_parse( formdata, formlist, 100 );

	p = cyg_formlist_find( formlist, "send_submit");
	if(p != NULL)
	{
		// for transmitting
		p = cyg_formlist_find( formlist, "std_id");
		if(p != NULL)
		{
			sscanf(p, "%x", &std_id);
			dirt = 1;
		}

		p = cyg_formlist_find( formlist, "can_len");
		if(p != NULL)
			sscanf(p, "%x", &can_len);

		p = cyg_formlist_find( formlist, "can_data0");
		if(p != NULL)
			sscanf(p, "%x", &can_data[0]);

		p = cyg_formlist_find( formlist, "can_data1");
		if(p != NULL)
			sscanf(p, "%x", &can_data[1]);

		p = cyg_formlist_find( formlist, "can_data2");
		if(p != NULL)
			sscanf(p, "%x", &can_data[2]);

		p = cyg_formlist_find( formlist, "can_data3");
		if(p != NULL)
			sscanf(p, "%x", &can_data[3]);

		p = cyg_formlist_find( formlist, "can_data4");
		if(p != NULL)
			sscanf(p, "%x", &can_data[4]);

		p = cyg_formlist_find( formlist, "can_data5");
		if(p != NULL)
			sscanf(p, "%x", &can_data[5]);

		p = cyg_formlist_find( formlist, "can_data6");
		if(p != NULL)
			sscanf(p, "%x", &can_data[6]);

		p = cyg_formlist_find( formlist, "can_data7");
		if(p != NULL)
			sscanf(p, "%x", &can_data[7]);

		if(dirt == 1)
		{
			// after reading all the input then need to send out a CAN message
			can_str_msg.std_id = (cyg_uint16)std_id;
			can_str_msg.length = (cyg_uint8)can_len;
			for(cnt=0;cnt<can_str_msg.length;cnt++)
			{
				can_str_msg.data[cnt]=(cyg_uint8)can_data[cnt];
			}
			api_can_send(can_str_msg);
		}
	}

	// look for the receive submit
	p = cyg_formlist_find( formlist, "recv_submit");
	if(p != NULL)
	{
//		diag_printf("Receive Submit Pressed!\n");
		// try to receive the requested can message
		p = cyg_formlist_find( formlist, "rstd_id");
		if(p != NULL)
			sscanf(p, "%x", &rstd_id);

		rcan_str_msg.std_id = rstd_id;
		if(api_can_recv(&rcan_str_msg, MODE_REMOVE) == CAN_ERROR)
			diag_printf("No Message!\n");
		else
		{
			// fill in the message
			rcan_len = rcan_str_msg.length;
			for(cnt=0;cnt<rcan_len;cnt++)
				rcan_data[cnt]=rcan_str_msg.data[cnt];
		}
	}

	// look for cofig submit
	p = cyg_formlist_find( formlist, "config_submit");
	if(p != NULL)
	{

		// cofig according to the user input
		p = cyg_formlist_find( formlist, "speed");
		if(p != NULL)
		{
			sscanf(p, "%x", &speed);
			switch(speed)
			{
			case 1:
				diag_printf("Set Speed: 1Mbps\n");
				cyg_api_can_set_speed(CAN_1M);
				break;

			case 2:
				diag_printf("Set Speed: 500Kbps\n");
				cyg_api_can_set_speed(CAN_500K);
				break;

			case 3:
				diag_printf("Set Speed: 250Kbps\n");
//				cyg_api_can_set_speed(CAN_250K);			
				break;

			case 4:
				diag_printf("Set Speed: 125Kbps\n");
				cyg_api_can_set_speed(CAN_125K);
				break;

			case 5:
				diag_printf("Set Speed: 50Kbps\n");
//				cyg_api_can_set_speed(CAN_50K);
				break;
			
			default:
				diag_printf("Set Speed: None\n");
				break;
			}
		}

		p = cyg_formlist_find( formlist, "mask");
		if(p != NULL)
		{
			sscanf(p, "%x", &mask);
			// set mask
			diag_printf("Set Mask: 0x%04x\n", mask);
			
			p = cyg_formlist_find( formlist, "filter");
			if(p != NULL)
			{
				sscanf(p, "%x", &filter);
				// set mask
				diag_printf("Set Filter: 0x%04x\n", filter);

				// set mask and filter
				cyg_api_can_set_mask_filter((cyg_uint16)mask, (cyg_uint16)filter);
			}
		}
	}

	// look for the Write to External Flash Memory Submit: wflash_submit
	p = cyg_formlist_find( formlist, "wflash_submit");
	if(p != NULL)
	{
		// look for the flash data content
		p = cyg_formlist_find( formlist, "flash_data");
		if(p != NULL)
			sscanf(p, "%s", &flash_data);
		diag_printf("Write Flash Data Message: %s!\n", flash_data);

		// look for the file name
		p = cyg_formlist_find( formlist, "fsfile_name");
		if(p != NULL)
			sscanf(p, "%s", &fsfile_name);
		diag_printf("Write Flash File Name Message: %s!\n", fsfile_name);
		
		// write the message to external flash memory

		diag_printf("Start Writing Flash memory file system!\n");

	}   
	
	// look for the Read from External Flash Memory Submit: rflash_submit
	p = cyg_formlist_find( formlist, "rflash_submit");
	if(p != NULL)
	{
		
		// look for the file name
		p = cyg_formlist_find( formlist, "fsfile_name");
		if(p != NULL)
			sscanf(p, "%s", &fsfile_name);
		diag_printf("Read Flash File Name Message: %s!\n", fsfile_name);

		// reload the message from external flash memory

	}    
		
	// look for the Write to RAM file system request: rram_submit
	p = cyg_formlist_find( formlist, "wram_submit");
	if(p != NULL)
	{
		diag_printf("Start Writing RAM file system!\n");
		
		// look for the file name
		p = cyg_formlist_find( formlist, "ramfile_name");
		if(p != NULL)
			sscanf(p, "%s", &ramfile_name);

		// look for the ram_data
		p = cyg_formlist_find( formlist, "ram_data");
		if(p != NULL)
			sscanf(p, "%s", &ram_data);

		diag_printf("RAM File Name: %s, Data: %s\n", ramfile_name, ram_data);
	}  
	
	// look for the Write to RAM file system request: rram_submit
	p = cyg_formlist_find( formlist, "rram_submit");
	if(p != NULL)
	{		
		diag_printf("Start Reading RAM file system!\n");

		// look for the file name
		p = cyg_formlist_find( formlist, "ramfile_name");
		if(p != NULL)
			sscanf(p, "%s", &ramfile_name);
		diag_printf("RAM File Name: %s\n", ramfile_name);		
	}  
	
	html_begin(client);

    html_head(client,"Ethernet <--> CAN Gateway", "");

    html_body_begin(client,"");
    {
        html_heading(client, 2, "Welcome to Ethernet CAN Gateway Website" );

        html_form_begin( client, "/gateway.html", "" );
        {
            html_para_begin( client, "" );			
            html_para_begin( client, "" );		
			fprintf(client, "Send CAN Message\n");
            html_para_begin( client, "" );
			// set CAN ID from User
            fprintf(client,
                    "CAN ID: 0x<input type=\"text\" name=\"std_id\" size=\"6\" value=\"%x\">\n",
                    std_id);

			// set the CAN message to be sent
            fprintf(client,
                    "CAN Data Length: 0x<input type=\"text\" name=\"can_len\" size=\"6\" value=\"%x\">\n",
                    can_len);

            /* Text input control for base address
             */
            html_para_begin( client, "" );
			// set CAN data to be sent or received
            fprintf(client,
                    "CAN Data[0]: 0x<input type=\"text\" name=\"can_data0\" size=\"5\" value=\"%x\">\n",
                    can_data[0]);
            fprintf(client,
                    "CAN Data[1]: 0x<input type=\"text\" name=\"can_data1\" size=\"5\" value=\"%x\">\n",
                    can_data[1]);
            fprintf(client,
                    "CAN Data[2]: 0x<input type=\"text\" name=\"can_data2\" size=\"5\" value=\"%x\">\n",
                    can_data[2]);
            fprintf(client,
                    "CAN Data[3]: 0x<input type=\"text\" name=\"can_data3\" size=\"5\" value=\"%x\">\n",
                    can_data[3]);
            fprintf(client,
                    "CAN Data[4]: 0x<input type=\"text\" name=\"can_data4\" size=\"5\" value=\"%x\">\n",
                    can_data[4]);
            fprintf(client,
                    "CAN Data[5]: 0x<input type=\"text\" name=\"can_data5\" size=\"5\" value=\"%x\">\n",
                    can_data[5]);
            fprintf(client,
                    "CAN Data[6]: 0x<input type=\"text\" name=\"can_data6\" size=\"5\" value=\"%x\">\n",
                    can_data[6]);
            fprintf(client,
                    "CAN Data[7]: 0x<input type=\"text\" name=\"can_data7\" size=\"5\" value=\"%x\">\n",
                    can_data[7]);
                    
			// new paragraph
            html_para_begin( client, "" );

			// Submit and reset buttons
            html_form_input(client, "submit", "send_submit", "Send", "");                
            html_form_input(client, "reset", "send_reset", "Reset", "");
        }
        html_form_end( client );

		html_form_begin( client, "/gateway.html", "" );
		{
            html_para_begin( client, "" );		
            html_para_begin( client, "" );		
			fprintf(client, "Receive CAN Message\n");
            html_para_begin( client, "" );

			// set CAN ID from User
            fprintf(client,
                    "Received CAN ID: 0x<input type=\"text\" name=\"rstd_id\" size=\"6\" value=\"%x\">\n",
                    rstd_id);

			// set the CAN message to be sent
            fprintf(client,
                    "CAN Data Length: 0x<input type=\"text\" name=\"rcan_len\" size=\"6\" value=\"%x\">\n",
                    rcan_len);

            /* Text input control for base address
             */
            html_para_begin( client, "" );
			// set CAN data to be sent or received
            fprintf(client,
                    "CAN Data[0]: 0x<input type=\"text\" name=\"rcan_data0\" size=\"5\" value=\"%x\">\n",
                    rcan_data[0]);
            fprintf(client,
                    "CAN Data[1]: 0x<input type=\"text\" name=\"rcan_data1\" size=\"5\" value=\"%x\">\n",
                    rcan_data[1]);
            fprintf(client,
                    "CAN Data[2]: 0x<input type=\"text\" name=\"rcan_data2\" size=\"5\" value=\"%x\">\n",
                    rcan_data[2]);
            fprintf(client,
                    "CAN Data[3]: 0x<input type=\"text\" name=\"rcan_data3\" size=\"5\" value=\"%x\">\n",
                    rcan_data[3]);
            fprintf(client,
                    "CAN Data[4]: 0x<input type=\"text\" name=\"rcan_data4\" size=\"5\" value=\"%x\">\n",
                    rcan_data[4]);
            fprintf(client,
                    "CAN Data[5]: 0x<input type=\"text\" name=\"rcan_data5\" size=\"5\" value=\"%x\">\n",
                    rcan_data[5]);
            fprintf(client,
                    "CAN Data[6]: 0x<input type=\"text\" name=\"rcan_data6\" size=\"5\" value=\"%x\">\n",
                    rcan_data[6]);
            fprintf(client,
                    "CAN Data[7]: 0x<input type=\"text\" name=\"rcan_data7\" size=\"5\" value=\"%x\">\n",
                    rcan_data[7]);
			// new paragraph
            html_para_begin( client, "" );

			// Submit and reset buttons
            html_form_input(client, "submit", "recv_submit", "Receive", "");                
            html_form_input(client, "reset", "recv_reset", "Reset", "");
		}
        html_form_end( client );

        html_form_begin( client, "/gateway.html", "" );
        {
		
            html_para_begin( client, "" );		
            html_para_begin( client, "" );		
			fprintf(client, "CAN Controller Configuration\n");
            html_para_begin( client, "" );
			// set CAN ID from User
            fprintf(client,
                    "Mask: 0x<input type=\"text\" name=\"mask\" size=\"6\" value=\"%x\">\n",
                    mask);

			// set the CAN message to be sent
            fprintf(client,
                    "Filter: 0x<input type=\"text\" name=\"filter\" size=\"6\" value=\"%x\">\n",
                    filter);

            fputs( "CAN Speed: ", client );
            html_form_select_begin( client, "speed", "" );
            html_form_option( client, "1", "1Mbps", speed==1 );
            html_form_option( client, "2", "500Kbps", speed==2 );
//            html_form_option( client, "3", "250Kbps", speed==3 );
            html_form_option( client, "4", "125Kbps", speed==4 );
//            html_form_option( client, "5", "50Kbps", speed==5 );
			html_form_select_end( client );
            html_para_begin( client, "" );

			// Submit and reset buttons
            html_form_input(client, "submit", "config_submit", "Config", "");                
            html_form_input(client, "reset", "config_reset", "Reset", "");
        }
        html_form_end( client );

        html_form_begin( client, "/gateway.html", "" );
        {

            html_para_begin( client, "" );		
            html_para_begin( client, "" );		
			fprintf(client, "Flash File System Accessing\n");
            html_para_begin( client, "" );		
            fprintf(client,
                    "Flash File Name: <input type=\"text\" name=\"fsfile_name\" size=\"10\" value=\"%s\">\n",
                    fsfile_name);

            fprintf(client,
                    "Flash Data: <input type=\"text\" name=\"flash_data\" size=\"10\" value=\"%s\">\n",
                    flash_data);

			// Submit and reset buttons
            html_form_input(client, "submit", "wflash_submit", "Write", "");                
            html_form_input(client, "submit", "rflash_submit", "Read", "");                
            html_form_input(client, "reset", "flash_reset", "Reset", "");
        }
        html_form_end( client );

        html_form_begin( client, "/gateway.html", "" );
        {

            html_para_begin( client, "" );		
            html_para_begin( client, "" );		
			fprintf(client, "RAM File System Accessing\n");
            html_para_begin( client, "" );		
            fprintf(client,
                    "RAM File Name: <input type=\"text\" name=\"ramfile_name\" size=\"10\" value=\"%s\">\n",
                    ramfile_name);

            fprintf(client,
                    "RAM Data: <input type=\"text\" name=\"ram_data\" size=\"10\" value=\"%s\">\n",
                    ram_data);

			// Submit and reset buttons
            html_form_input(client, "submit", "wram_submit", "Write", "");                
            html_form_input(client, "submit", "rram_submit", "Read", "");                
            html_form_input(client, "reset", "ram_reset", "Reset", "");
        }
        html_form_end( client );

    }
    html_body_end(client);

    html_end(client);
    
    return 1;
}


CYG_HTTPD_TABLE_ENTRY( cyg_web_server_entry,
                       "/gateway.htm*",
                       cyg_web_server,
                       NULL );



void
cyg_start(void)
{
	// should initialise the interface
    init_all_network_interfaces();

	// start the scheduler
	cyg_scheduler_start();

}
