#ifndef CYGONCE_EPXA1_H
#define CYGONCE_EPXA1_H

//=============================================================================
//
//      epxa1.h
//
//      EPXA1 Platform specific support (register layout, etc)
//
//=============================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    Qiang Huang
// Contributors: None
// Date:         2003-01-18
// Purpose:      Altera/EPXA1 platform specific support routines
// Description: 
// Usage:        #include <cyg/hal/epxa1.h>
//
//####DESCRIPTIONEND####
//
//=============================================================================

#include <pkgconf/hal_arm_arm9_epxa1.h>

#define EPXA1_BASE         EPXA1_REGS_PHYS_BASE

//-----------------------------------------------------------------------------
// Boot control
// Note: this register is actually write-bit-to-clear-it
#define EPXA1_BOOT_CR      (EPXA1_BASE + 0x0000)
// EBI0(32Kbytes) mapped at address 0x00
#define EPXA1_BOOT_CR_BM   0x00000001
// uP hold in reset mode
#define EPXA1_BOOT_CR_HM   0x00000002
// register enable
#define EPXA1_BOOT_CR_RE   0x00000004

//-----------------------------------------------------------------------------
// DPSRAM config -- EPXA1 only have block 0 DPSRAM EPXA1 -- 32Kbytes SPRAM, 16Kbytes DPRAM
#define EPXA1_DPSRAM_BASE  (EPXA1_BASE + 0x0030)
#define _DPSRAM0_SR            0x0000

// Question:  What is 34H ?? "LCR"??
#define _DPSRAM0_LCR           0x0004

#define _DPSRAM0_LCR_INIT      0x00000000

//-----------------------------------------------------------------------------
// IO controller
#define EPXA1_IOCR_BASE    (EPXA1_BASE + 0x0040)
#define _IOCR_SDRAM            0x0000
#define _IOCR_EBI              0x0004
#define _IOCR_UART             0x0008
#define _IOCR_TRACE            0x000c

#define _IOCR_OC_PCI           0x00000008
#define _IOCR_OC_FAST          0x00000004
#define _IOCR_OC_SLOW          0x00000000
#define _IOCR_IO_STRIPE        0x00000002
#define _IOCR_LOCK             0x00000001

#define EPXA1_IOCR_SDRAM_INIT (_IOCR_OC_FAST | _IOCR_IO_STRIPE | _IOCR_LOCK)
#define EPXA1_IOCR_EBI_INIT   (_IOCR_OC_SLOW | _IOCR_IO_STRIPE | _IOCR_LOCK)
#define EPXA1_IOCR_UART_INIT  (_IOCR_OC_SLOW | _IOCR_IO_STRIPE | _IOCR_LOCK)


//-----------------------------------------------------------------------------
// Memory mapping
#define EPXA1_MMAP_BASE    (EPXA1_BASE + 0x0080)
#define EPXA1_MMAP_EBI3	   (EPXA1_BASE + 0x0cc)
#define EPXA1_MMAP_EBI2	   (EPXA1_BASE + 0x0c8)
#define EPXA1_MMAP_EBI1	   (EPXA1_BASE + 0x0c4)
#define EPXA1_MMAP_EBI0	   (EPXA1_BASE + 0x0c0)

#define _MMAP_REGISTERS        0x0000
#define _MMAP_SRAM0            0x0010
#define _MMAP_SRAM1            0x0014
#define _MMAP_DPSRAM0          0x0020
#define _MMAP_SDRAM0           0x0030
#define _MMAP_EBI0             0x0040
#define _MMAP_EBI1             0x0044
#define _MMAP_EBI2             0x0048
#define _MMAP_EBI3             0x004c
#define _MMAP_PLD0             0x0050
#define _MMAP_PLD1             0x0054
#define _MMAP_PLD2             0x0058
#define _MMAP_PLD3             0x005c

#define _MMAP_SIZE_16K   (13<<7)
#define _MMAP_SIZE_64K   (15<<7)
#define _MMAP_SIZE_128K  (16<<7)
#define _MMAP_SIZE_1M    (19<<7)
#define _MMAP_SIZE_4M    (21<<7)
#define _MMAP_SIZE_16M   (23<<7)
#define _MMAP_SIZE_32M   (24<<7)
#define _MMAP_SIZE_64M   (25<<7)

#define _MMAP_PREFETCH         0x00000000
#define _MMAP_NOPREFETCH       0x00000002

#define _MMAP_ENABLE           0x00000001
#define _MMAP_DISABLE          0x00000000

#define _MMAP_REGISTERS_INIT   (EPXA1_REGS_PHYS_BASE  + 0x00000000 | _MMAP_SIZE_16K | _MMAP_NOPREFETCH | _MMAP_ENABLE)
#define _MMAP_SRAM0_INIT       (EPXA1_SRAM_PHYS_BASE  + 0x00000000 | _MMAP_SIZE_16K | _MMAP_PREFETCH   | _MMAP_ENABLE)
#define _MMAP_SRAM1_INIT       (EPXA1_SRAM_PHYS_BASE  + 0x00004000 | _MMAP_SIZE_16K | _MMAP_PREFETCH   | _MMAP_ENABLE)
#define _MMAP_DPSRAM0_INIT     (EPXA1_SRAM_PHYS_BASE  + 0x00008000 | _MMAP_SIZE_16K | _MMAP_PREFETCH   | _MMAP_ENABLE)
#define _MMAP_SDRAM0_INIT      (EPXA1_SDRAM_PHYS_BASE + 0x00000000 | _MMAP_SIZE_32M | _MMAP_PREFETCH   | _MMAP_ENABLE)
// EPXA1 only have SDRAM0, so not map SDRAM1
//#define _MMAP_SDRAM1_INIT      (EPXA1_SDRAM_PHYS_BASE + 0x04000000 | _MMAP_SIZE_64M  | _MMAP_PREFETCH   | _MMAP_ENABLE)
// EPXA1 only have 8Mbytes flash memory and each 4Mbytes allocated for EBI0 and EBI1
#define _MMAP_EBI0_INIT        (EPXA1_FLASH_PHYS_BASE + 0x00000000 | _MMAP_SIZE_4M   | _MMAP_PREFETCH   | _MMAP_ENABLE)
#define _MMAP_EBI1_INIT        (EPXA1_FLASH_PHYS_BASE + 0x00400000 | _MMAP_SIZE_4M   | _MMAP_PREFETCH   | _MMAP_ENABLE)
#define _MMAP_EBI2_INIT        (EPXA1_FLASH_PHYS_BASE + 0x00800000 | _MMAP_SIZE_4M   | _MMAP_PREFETCH   | _MMAP_ENABLE)
// Ethernet controller here
#define _MMAP_EBI3_INIT        (EPXA1_FLASH_PHYS_BASE + 0x00c00000 | _MMAP_SIZE_4M  | _MMAP_NOPREFETCH | _MMAP_ENABLE)
#define _MMAP_PLD0_INIT        (EPXA1_PLD_PHYS_BASE   + 0x00000000 | _MMAP_SIZE_16K | _MMAP_NOPREFETCH | _MMAP_ENABLE)
//#define _MMAP_PLD1_INIT        (EXCALIBUR_PLD_PHYS_BASE   + 0x00004000 | _MMAP_SIZE_16K | _MMAP_NOPREFETCH | _MMAP_ENABLE)

// Question: Why 0x0f000000 other than EPXA1_PLD_PHYS_BASE??
//#define _MMAP_PLD1_INIT        (0x0f000000 | _MMAP_SIZE_16K | _MMAP_NOPREFETCH | _MMAP_ENABLE)
//#define _MMAP_PLD2_INIT        (EPXA1_PLD_PHYS_BASE   + 0x00008000 | _MMAP_SIZE_16K | _MMAP_NOPREFETCH | _MMAP_ENABLE)
//#define _MMAP_PLD3_INIT        (EPXA1_PLD_PHYS_BASE   + 0x0000c000 | _MMAP_SIZE_16K | _MMAP_NOPREFETCH | _MMAP_ENABLE)

#define EPXA1_SDRAM_PHYS_BASE 0x00000000
#define EPXA1_FLASH_PHYS_BASE 0x40000000
#define EPXA1_SRAM_PHYS_BASE  0x08000000
#define EPXA1_PLD_PHYS_BASE   0x80000000
#define EPXA1_REGS_PHYS_BASE  0x7fffc000

// On-Board Ethernet (physical addrs) 
#define	EXC_EBI_BLOCK3_BASE   (0x40c00000)
#define ALTERA_EPXA1_ETH_IOBASE       (EXC_EBI_BLOCK3_BASE + 0x300)	// EBI3 -- IO base address
#define EPXA1_IRQ_GPIO_ETH	CYGNUM_HAL_INTERRUPT_EXTERNAL

//-----------------------------------------------------------------------------
// Timers
#define EPXA1_TIMER0_CR              (EPXA1_BASE+0x0200)
#define EPXA1_TIMER0_PRE             (EPXA1_BASE+0x0210)
#define EPXA1_TIMER0_LIMIT           (EPXA1_BASE+0x0220)
#define EPXA1_TIMER0_READ            (EPXA1_BASE+0x0230)

#define EPXA1_TIMER1_CR              (EPXA1_BASE+0x0240)
#define EPXA1_TIMER1_PRE             (EPXA1_BASE+0x0250)
#define EPXA1_TIMER1_LIMIT           (EPXA1_BASE+0x0260)
#define EPXA1_TIMER1_READ            (EPXA1_BASE+0x0270)

#define EPXA1_TIMER_CR_MODE_HEARBEAT 0x00000000
#define EPXA1_TIMER_CR_MODE_ONE_SHOT 0x00000001
#define EPXA1_TIMER_CR_IE            0x00000004
#define EPXA1_TIMER_CR_CI            0x00000008
#define EPXA1_TIMER_CR_S             0x00000010

//-----------------------------------------------------------------------------
// Serial
#define EPXA1_UART0_BASE       (EPXA1_BASE+0x0280)
#define _UART_RSR              0x0000
#define _UART_RDS              0x0004
#define _UART_RD               0x0008
#define _UART_TSR              0x000c
#define _UART_TD               0x0010
#define _UART_FCR              0x0014
#define _UART_IES              0x0018
#define _UART_IEC              0x001c
#define _UART_ISR              0x0020
#define _UART_IID              0x0024
#define _UART_MC               0x0028
#define _UART_MCR              0x002c
#define _UART_MSR              0x0030
#define _UART_DIV_LO           0x0034
#define _UART_DIV_HI           0x0038

#define _UART_RSR_RX_LEVEL     0x0000001f

#define _UART_TSR_TXI          0x00000080

#define _UART_FCR_TC           0x00000001
#define _UART_FCR_RC           0x00000002
#define _UART_FCR_TX_THR_15    0x0000001c
#define _UART_FCR_RX_THR_1     0x00000000

#define _UART_INTS_RE          0x00000001
#define _UART_INTS_RI          _UART_INTS_RE
#define _UART_INTS_TE          0x00000002
#define _UART_INTS_TI          _UART_INTS_TE
#define _UART_INTS_TIE         0x00000004
#define _UART_INTS_TII         _UART_INTS_TIE
#define _UART_INTS_ME          0x00000008
#define _UART_INTS_MI          _UART_INTS_ME


#define _UART_MC_8BIT          0x00000003
#define _UART_MC_1STOP         0x00000000
#define _UART_MC_PARITY_NONE   0x00000000

//-----------------------------------------------------------------------------
// Clock controller
#define EPXA1_CLK_BASE     (EPXA1_BASE + 0x0300)
#define _CLK_PLL1_NCNT         0x0000
#define _CLK_PLL1_MCNT         0x0004
#define _CLK_PLL1_KCNT         0x0008
#define _CLK_PLL1_CTRL         0x000c
#define _CLK_PLL2_NCNT         0x0010
#define _CLK_PLL2_MCNT         0x0014
#define _CLK_PLL2_KCNT         0x0018
#define _CLK_PLL2_CTRL         0x001c
#define _CLK_DERIVE            0x0020
#define _CLK_STATUS            0x0024
#define _CLK_AHB1_COUNT        0x0028

#define _CLK_PLL1_CTRL_P    0x00000001
#define _CLK_PLL2_CTRL_P    0x00000001

#define _CLK_DERIVE_BP1     (1<<12)
#define _CLK_DERIVE_BP2     (1<<13)

#define _CLK_STATUS_L1       0x00000001
#define _CLK_STATUS_L2       0x00000002
#define _CLK_STATUS_C1       0x00000004
#define _CLK_STATUS_C2       0x00000008

//-----------------------------------------------------------------------------
// Expansion Bus Interface
#define EPXA1_EBI_CR       (EPXA1_BASE + 0x0380)
#define EPXA1_EBI_INT_ADDRSR	(EPXA1_BASE + 0x03A4)
#define EPXA1_EBI_BLOCK3   (EPXA1_BASE + 0x039c)

// enable the clock output for the ethernet device smc91c111
#define EPXA1_EBI_CR_CE_EO    0x00080008

#define EPXA1_EBI_CR_INIT  (EPXA1_EBI_CR_CE_EO)
// maximum wait state for external ethernet device
#define _EPXA1_EBI_BLOCK3_INIT  	0x9e

//-----------------------------------------------------------------------------
// SDRAM controller address
#define EPXA1_SDRAM_BASE   (EPXA1_BASE + 0x0400)
#define _SDRAM_TIMING1         0x0000
#define _SDRAM_TIMING2         0x0004
#define _SDRAM_CONFIG          0x0008
#define _SDRAM_REFRESH         0x000c
#define _SDRAM_ADDR            0x0010
#define _SDRAM_INIT            0x001c
#define _SDRAM_MODE0           0x0020
#define _SDRAM_MODE1           0x0024
#define EPXA1_SDRAM_WIDTH  (EPXA1_BASE + 0x007c)

// SDRAM on the EPXA1 is 16-bit
#define _SDRAM_WIDTH_W        0x00000000
#define _SDRAM_WIDTH_LK       0x00000001
#define _SDRAM_INIT_EN      0x00008000
#define _SDRAM_INIT_PR      0x00004000
#define _SDRAM_INIT_LM      0x00002000
#define _SDRAM_INIT_LEM     0x00001000
#define _SDRAM_INIT_RF      0x00000800
#define _SDRAM_INIT_BS      0x00000400
#define _SDRAM_INIT_SR      0x00000200

#if (CYGNUM_HAL_ARM_EPXA1_SDRAM_CLOCK != 100000000)
# error "Hardwired for a 100MHz SDRAM clock"
#endif

//-----------------------------------------------------------------------------
// Watchdog controller
#define EPXA1_WDOG_CR               (EPXA1_BASE+0x0a00)
#define EPXA1_WDOG_COUNT            (EPXA1_BASE+0x0a04)
#define EPXA1_WDOG_RELOAD           (EPXA1_BASE+0x0a08)

//-----------------------------------------------------------------------------
// Interrupt controller
#define EPXA1_INT_MASK_SET          (EPXA1_BASE+0x0c00)
#define EPXA1_INT_MASK_CLEAR        (EPXA1_BASE+0x0c04)
#define EPXA1_INT_SOURCE_STATUS     (EPXA1_BASE+0x0c08)
#define EPXA1_INT_REQUEST_STATUS    (EPXA1_BASE+0x0c0c)
#define EPXA1_INT_ID                (EPXA1_BASE+0x0c10)
#define EPXA1_INT_PLD_PRIORITY      (EPXA1_BASE+0x0c14)
#define EPXA1_INT_INT_MODE          (EPXA1_BASE+0x0c18)
#define EPXA1_INT_PRIORITY_0        (EPXA1_BASE+0x0c80)
#define EPXA1_INT_PRIORITY_1        (EPXA1_BASE+0x0c84)
#define EPXA1_INT_PRIORITY_2        (EPXA1_BASE+0x0c88)
#define EPXA1_INT_PRIORITY_3        (EPXA1_BASE+0x0c8c)
#define EPXA1_INT_PRIORITY_4        (EPXA1_BASE+0x0c90)
#define EPXA1_INT_PRIORITY_5        (EPXA1_BASE+0x0c94)
#define EPXA1_INT_PRIORITY_6        (EPXA1_BASE+0x0c98)
#define EPXA1_INT_PRIORITY_7        (EPXA1_BASE+0x0c9c)
#define EPXA1_INT_PRIORITY_8        (EPXA1_BASE+0x0ca0)
#define EPXA1_INT_PRIORITY_9        (EPXA1_BASE+0x0ca4)
#define EPXA1_INT_PRIORITY_10       (EPXA1_BASE+0x0ca8)
#define EPXA1_INT_PRIORITY_11       (EPXA1_BASE+0x0cac)
#define EPXA1_INT_PRIORITY_12       (EPXA1_BASE+0x0cb0)
#define EPXA1_INT_PRIORITY_13       (EPXA1_BASE+0x0cb4)
#define EPXA1_INT_PRIORITY_14       (EPXA1_BASE+0x0cb8)
#define EPXA1_INT_PRIORITY_15       (EPXA1_BASE+0x0cbc)
#define EPXA1_INT_PRIORITY_16       (EPXA1_BASE+0x0cc0)

#define EPXA1_INT_SOURCE_P0         0x00000001
#define EPXA1_INT_SOURCE_P1         0x00000002
#define EPXA1_INT_SOURCE_P2         0x00000004
#define EPXA1_INT_SOURCE_P3         0x00000008
#define EPXA1_INT_SOURCE_P4         0x00000010
#define EPXA1_INT_SOURCE_P5         0x00000020
#define EPXA1_INT_SOURCE_IP         0x00000040
#define EPXA1_INT_SOURCE_UA         0x00000080
#define EPXA1_INT_SOURCE_T0         0x00000100
#define EPXA1_INT_SOURCE_T1         0x00000200
#define EPXA1_INT_SOURCE_PS         0x00000400
#define EPXA1_INT_SOURCE_EE         0x00000800
#define EPXA1_INT_SOURCE_PE         0x00001000
#define EPXA1_INT_SOURCE_AE         0x00002000
#define EPXA1_INT_SOURCE_CT         0x00004000
#define EPXA1_INT_SOURCE_CR         0x00008000
#define EPXA1_INT_SOURCE_FC         0x00010000

#define EPXA1_INT_PRIORITY_FIQ      0x00000040
#define EPXA1_INT_PRIORITY_LVL_mask 0x0000003f

//-----------------------------------------------------------------------------
// PLD
//#define EPXA1_PLD_BASE     EPXA1_PLD_PHYS_BASE

//#define EPXA1_PLD_LEDS     (EPXA1_PLD_BASE + 0x0100)
#define EPXA1_CONFIG_CLOCK	(EPXA1_BASE+0x0144)
#define EPXA1_CONFIG_CRTL	(EPXA1_BASE+0x0140)
#define EPXA1_IDCODE		(EPXA1_BASE+0x0008)
#define EPXA1_CONFIG_DATA	(EPXA1_BASE+0x0148)

#endif // CYGONCE_EPXA1_H
//-----------------------------------------------------------------------------
// end of epxa1.h
