#ifndef CYGONCE_HAL_PLATFORM_SETUP_H
#define CYGONCE_HAL_PLATFORM_SETUP_H
//=============================================================================
//
//      hal_platform_setup.h
//
//      Platform specific support for HAL (assembly code)
//
//=============================================================================
//=============================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    Qiang Huang
// Contributors: None
// Date:         2003-01-22
// Purpose:      ARM9/EPXA1 platform specific support routines
// Description: 
// Usage:        #include <cyg/hal/hal_platform_setup.h>
//     Only used by "vectors.S"         
//
//####DESCRIPTIONEND####
//
//=============================================================================

#include <pkgconf/system.h>             // System-wide configuration info
#include CYGBLD_HAL_VARIANT_H           // Variant specific configuration
#include CYGBLD_HAL_PLATFORM_H          // Platform specific configuration
#include CYGHWR_MEMORY_LAYOUT_H
#include <cyg/hal/hal_mmu.h>            // MMU definitions
#include <cyg/hal/epxa1.h>          // Platform specific hardware definitions

#define nDEBUG
#define n_ALTERA_CACHEHACK  // doesn't have any apparent effect

// only take effect when target startup: ROMRAM, ROM, REDBOOT doesn't affect RAM startup
// need to setup the memory map, setup peripheral(PLL, UART, Timer.. etc.) coz no one do this yet. if for RAM startup the OS(or, Redboot, bootstraloader etc) load this image has
// already done this procedure

#if defined(CYG_HAL_STARTUP_ROM) || defined(CYG_HAL_STARTUP_ROMRAM) || defined(CYG_HAL_STARTUP_REDBOOT)
#define PLATFORM_SETUP1 _platform_setup1
#define CYGHWR_HAL_ARM_HAS_MMU
#define CYGSEM_HAL_ROM_RESET_USES_JUMP

#define CYG_DEVICE_SERIAL_BAUD_DIV (CYGNUM_HAL_ARM_EPXA1_PERIPHERAL_CLOCK/CYGNUM_HAL_VIRTUAL_VECTOR_CONSOLE_CHANNEL_BAUD/16)
#define CYG_DEVICE_SERIAL_BAUD_LSB (CYG_DEVICE_SERIAL_BAUD_DIV&0xff)
#define CYG_DEVICE_SERIAL_BAUD_MSB ((CYG_DEVICE_SERIAL_BAUD_DIV>>8)&0xff)

// We need this here - can't rely on a translation table until MMU has
// been initialized
// 0x41 -- 'A' 
        .macro RAW_LED_MACRO x
#ifdef DEBUG
        ldr     r0,=(EPXA1_UART0_BASE+_UART_TD)
        mov     r1,#(\x + 0x41)
        str     r1,[r0]
#endif
        .endm

// This macro represents the initial startup code for the platform        
        .macro  _platform_setup1
        // IO controller init
        ldr     r1,=EPXA1_IOCR_BASE
        ldr     r2,=EPXA1_IOCR_SDRAM_INIT
        str     r2,[r1,#_IOCR_SDRAM]
        ldr     r2,=EPXA1_IOCR_EBI_INIT
        str     r2,[r1,#_IOCR_EBI]
        ldr     r2,=EPXA1_IOCR_UART_INIT
        str     r2,[r1,#_IOCR_UART]

        // Expansion bus init
        ldr     r1,=EPXA1_EBI_CR
        ldr     r2,=EPXA1_EBI_CR_INIT
        str     r2,[r1]

#ifdef DEBUG
        // Init UART for debug tracing
        ldr     r4,=EPXA1_UART0_BASE
        ldr     r2,=(_UART_MC_8BIT | _UART_MC_1STOP | _UART_MC_PARITY_NONE)
        str     r2,[r4,#_UART_MC]
        ldr     r2,=CYG_DEVICE_SERIAL_BAUD_LSB
        str     r2,[r4,#_UART_DIV_LO]
        ldr     r2,=CYG_DEVICE_SERIAL_BAUD_MSB
        str     r2,[r4,#_UART_DIV_HI]
        ldr     r2,=(_UART_FCR_TC | _UART_FCR_RC | _UART_FCR_TX_THR_15 | _UART_FCR_RX_THR_1)
        str     r2,[r4,#_UART_FCR]
#endif

        // Setup the PLLs see the label PLL_ADDR below for the input
        // clock frequency and the desired output frequencies of PLL1
        // and PLL2 Load the value into K,M,N for PLL 1 and 2
        adr     r0,2f
        ldmia   r0,{r0-r11}
        str     r6,[r0]
        str     r7,[r1]
        str     r8,[r2]
        str     r9,[r3]
        str     r10,[r4]
        str     r11,[r5]
                
        // Turn on the PLLs
        ldr     r3,=EPXA1_CLK_BASE
		// 2.5ns lock window(CLK_REF/N=25MHz/1=25MHz>20MHz => min 1.0ns) , low power mode disabled, PLL enabled
        ldr     r1,=(0x1055 | _CLK_PLL1_CTRL_P)
        str     r1,[r3, #_CLK_PLL1_CTRL]
        str     r1,[r3, #_CLK_PLL2_CTRL]

        // Ensure the PLLs are not in bypass
        ldr     r1,=(0x10 | _CLK_DERIVE_BP1 | _CLK_DERIVE_BP2)
//      str     r1,[r3, #_CLK_DERIVE]
//      orr     r1,r1,#0x300 /* Use PLL2 for AHB and for the SDRAM */
//      str     r1,[r3, #_CLK_DERIVE]
        ldr     r2,=(_CLK_DERIVE_BP1 | _CLK_DERIVE_BP2)
        // clear the bypass bits (bit clear)
		bic     r1,r1,r2
        str     r1,[r3, #_CLK_DERIVE]

        // Poll waiting for the PLL's to lock and the bits to not be
        // in bypass mode
        ldr     r2,= _CLK_STATUS_L1 | _CLK_STATUS_L2 // _CLK_STATUS_L2
1:      ldr     r1, [r3, #_CLK_STATUS]
        and     r1, r1, r2
        cmp     r1, r2
        bne     1b

        // Clear the interrupt caused by the change in lock status
        ldr     r2, =(_CLK_STATUS_C1 | _CLK_STATUS_C2)
        str     r2, [r3, #_CLK_STATUS]
        
        b       3f

        // PLL Registers Addresses
2:      .long EPXA1_CLK_BASE+_CLK_PLL1_KCNT
        .long EPXA1_CLK_BASE+_CLK_PLL1_MCNT
        .long EPXA1_CLK_BASE+_CLK_PLL1_NCNT
        .long EPXA1_CLK_BASE+_CLK_PLL2_KCNT
        .long EPXA1_CLK_BASE+_CLK_PLL2_MCNT
        .long EPXA1_CLK_BASE+_CLK_PLL2_NCNT
        // PLL Registers Values ensure this follows on from the
        // addresses the code depends on it
		// EPXA1
		// CLK_REF = 25MHz
		// F(AHB1,uP) = (CLK_REF*M)/(N*K*2) = 150MHz
		// F(SDRAM) = (CLK_REF*M)/(N*K*2) = 100MHz
        .long 0x40000 // CLK_PLL1_KCNT_VAL  = 1
        .long 0x20606 // CLK_PLL1_MCNT_VAL  = 12
        .long 0x40000 // CLK_PLL1_NCNT_VAL  = 1
        .long 0x40000 // CLK_PLL2_KCNT_VAL  = 1
        .long 0x20404 // CLK_PLL2_MCNT_VAL  = 8
        .long 0x40000 // CLK_PLL2_NCNT_VAL  = 1
3:

        RAW_LED_MACRO 0	// 'A'

        // Jump to ROM, becase this is target for ROM, ROMRAM, REDBOOT; so should startup from Flash0(EBI0)
		// after remap still need to continue the code in flash so adjustment to the pc is needed after remapping EBI0 to 0x40000000H
		// remap the Flash to address after the SDRAM as:
		// SDRAM:  0x00000000 ~ 0x3fffffff
		// FLASH0(EBI0): 0x40000000 ~ 0x43fffff -- 4MBytes
		// FLASH1: 0x44000000 ~ 0x47fffff -- 4MBytes
		// PLD: 0x80000000 16KBytes * 4 PLDs
        ldr     r1,=(EPXA1_MMAP_BASE + _MMAP_EBI0)
        ldr     r2,=_MMAP_EBI0_INIT
        str     r2,[r1]
		// Note: although EBI0 has been remapped to address 0x40000000, but it is still accessible via an alias
		// at address 0x00H (first 32Kbytes) according the Boot control register is set at power up. 
		// writing a "1" to bit0 of Boot_CR register will clear the alias mapping.
        ldr     r1,=CYGMEM_REGION_rom
		// jump to the next instruction in FLASH memory after remapped
        add     pc,pc,r1
        nop
        nop

        // writing '1' to bit0 of Boot_CR register will disable ROM mapping (alias mapping of EBI0 (first 32Kbytes) to 0x00)
        ldr     r1,=EPXA1_BOOT_CR
        ldr     r2,=EPXA1_BOOT_CR_BM
        str     r2,[r1]

        RAW_LED_MACRO 1	// 'B'

		// access the co-processor #15 for controlling the MMU and setting the Cache_Data_Instruction
        // Disable and clear caches
        
        mrc  p15,0,r0,c1,c0,0
        bic  r0,r0,#0x1000              // disable ICache
        bic  r0,r0,#0x0007              // disable DCache,
                                        // MMU and alignment faults
        mcr  p15,0,r0,c1,c0,0
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        mov  r0,#0
        mcr  p15,0,r0,c7,c6,0           // clear data cache
        mcr  p15,0,r0,c7,c5,0           // clear instruction cache
        
#if 0
        mrc  p15,0,r0,c15,c1,0          // disable streaming
        orr  r0,r0,#0x80
        mcr  p15,0,r0,c15,c1,0
#endif

        RAW_LED_MACRO 2	// 'C'

        // Set memory mapping
		// register memory map 16KBytes
        ldr     r1,=EPXA1_MMAP_BASE
        ldr     r2,=_MMAP_REGISTERS_INIT
        str     r2,[r1,#_MMAP_REGISTERS]

		// SRAM0 memory map
        ldr     r2,=_MMAP_SRAM0_INIT
        str     r2,[r1,#_MMAP_SRAM0]

		// SRAM1 memory map
        ldr     r2,=_MMAP_SRAM1_INIT
        str     r2,[r1,#_MMAP_SRAM1]

		// DPSRAM0 memory map
        ldr     r2,=_MMAP_DPSRAM0_INIT
        str     r2,[r1,#_MMAP_DPSRAM0]

		// Only 1 DPSRAM in EPXA1
//      ldr     r2,=_MMAP_DPSRAM1_INIT
//      str     r2,[r1,#_MMAP_DPSRAM1]

		// SDRAM0 memory map (32MBytes)
        ldr     r2,=_MMAP_SDRAM0_INIT
        str     r2,[r1,#_MMAP_SDRAM0]

		// Only 32MBytes SDRAM in SDRAM0 on EPXA1 board
//      ldr     r2,=_MMAP_SDRAM1_INIT
//      str     r2,[r1,#_MMAP_SDRAM1]

		// PLD memory map
        ldr     r2,=_MMAP_PLD0_INIT
        str     r2,[r1,#_MMAP_PLD0]
//      ldr     r2,=_MMAP_PLD1_INIT
//      str     r2,[r1,#_MMAP_PLD1]
//      ldr     r2,=_MMAP_PLD2_INIT
//      str     r2,[r1,#_MMAP_PLD2]
//      ldr     r2,=_MMAP_PLD3_INIT
//      str     r2,[r1,#_MMAP_PLD3]

		// EBI1 memory map
		// (EBI0 already mapped previously)
       ldr     r2,=_MMAP_EBI1_INIT
       str     r2,[r1,#_MMAP_EBI1]
       ldr     r2,=_MMAP_EBI2_INIT
       str     r2,[r1,#_MMAP_EBI2]
	   // Ethernet Controller on EBI3 bank
       ldr     r2,=_MMAP_EBI3_INIT
       str     r2,[r1,#_MMAP_EBI3]

	   // config EBI3 
       ldr     r1,=EPXA1_EBI_BLOCK3
       ldr     r2,=_EPXA1_EBI_BLOCK3_INIT
       str     r2,[r1]

        RAW_LED_MACRO 3	// 'D'

        // FIXME: Disable MMAP registers?

        RAW_LED_MACRO 4	// 'E'

        // DPSRAM init
		// Question: where is this register(xx_LCR) in EPXA1 ??
		// should not needed here
//      ldr     r2,=EPXA1_DPSRAM_BASE
//      ldr     r1,=_DPSRAM0_LCR_INIT
//      str     r1,[r2,#_DPSRAM0_LCR]

//      ldr     r1,=_DPSRAM1_LCR_INIT
//      str     r1,[r2,#_DPSRAM1_LCR]

        RAW_LED_MACRO 5	// 'F'

#ifdef _ALTERA_CACHEHACK
        mrc     p15,0,r0,c1,c0,0
        orr     r0,r0,#0x1000              // enable ICache
        mcr     p15,0,r0,c1,c0,0
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
#endif //  _ALTERA_CACHEHACK

        //  Setup the SDR

        // All the clock values below assume the SDR is running @ 100 MHz, CPU = 150MHz

        // 1. Wait for the PLL's to lock this was already done in
        // Setup_plls Then wait another 100uS, which given we're
        // running @ 150MHz is 15,000 clock cycles
        mov     r3,#0x3b00 // (15104)
1:      subs    r3,r3,#1
        bne     1b


        // 2. Set up all the SDRAM Controllers configuration settings
        // These are done using LDMIA's as it's more efficient, they're
        // done in the order they appear in the datasheet with the
        // exception of SDRAM_INIT which has to be written last

        // Load the address of the first seven SDRAM registers and the
        // contents
		// adr -- pseudo-instruction, store the address of SDRAM_REGS_ADDR into register r0, should be within 1020bytes
        adr     r0,SDRAM_REGS_ADDR
		// r0 = _SDRAM_TIMING1
		// r1 = _SDRAM_TIMING2
		// r2 = _SDRAM_CONFIG
		// r3 = _SDRAM_REFRESH
		// r4 = _SDRAM_ADDR
		// r5 = _SDRAM_MODE0
		//    = _SDRAM_MODE1
        ldmia   r0,{r0-r5}

		// get the address of SDRAM_REGS_VALUE
        adr     r6,SDRAM_REGS_VALUE

		// MT48LC16M16A2
		//======================================================================================================
		// RCD = (min: 15ns) => 15ns*100MHz/1000 = 1.5 clocks => 4 clocks => 0b100
		// RAS = (min: 37ns) => 37ns*100MHz/1000 = 3.7 clocks => 8 clocks => 0b1000
		// RRD = (min: 14ns) => 14ns*100MHz/1000 = 1.4 clocks => 3 clocks => 0b011
		// RP  = (min: 15ns) => 15ns*100MHz/1000 = 1.5 clocks => 3 clocks => 0b011
		// WR  = (min: 14ns; 1clk+7ns=10ns+7ns=17ns) => 17ns*100MHz/1000 = 1.7 clocks => 3 clocks => 0b011
		// r6 = SDRAM_TIMING1_VAL = 0x90db
		//======================================================================================================

		
		//======================================================================================================
		// RC = (min: 60ns) => 60ns*100MHz/1000 = 6 clocks => 8 clocks => 0b1000
		// CL = (CAS latency) = 2 clocks => 011
		// BL = (Burst Length) = 8 beat bursts => 11 ==> 0b11
		// RFC = (min: 66ns) = 66ns*100MHz/1000 = 6.6 clocks => 8 clocks => 0b1000
		// 0b000
		// r7 = SDRAM_TIMING2_VAL = 0x87c0
		//======================================================================================================

		
		//======================================================================================================
		// MT = 0 select SDR SDRAM
		// r8 = SDRAM_CONFIG = 0x00000000
		//======================================================================================================


		//======================================================================================================
		// RFSH = (SDRAM hidden refresh period)
		// 8192 auto refresh cycles each 64ms, 7.81us/auto refresh; so between each auto
		// cycles: 781 clocks, to meet this minimum cycle rate 781 clocks is the maximum clocks ==> 700 clocks 
		// = 0x02bc
		// r9 = SDRAM_REFRESH_VAL = 0x02bc
		//======================================================================================================


		//======================================================================================================
		// ROW = 13 bits => 0x0d = 0b1101
		// COL = 9 bits = 0x09 = 0b1001
		// 0b10
		// r10 = SDRAM_ADDR_VAL = 0xd980
		//======================================================================================================

	
		//======================================================================================================
		// 0b000
		// programmed burst length(WB) => 0b0
		// Op Mode = Standard => 0b00 
		// CAS Latency = 2 clocks => 0b010
		// BT = Sequential => 0b0
		// Burst Length = 8 => 0b011
		// r11 = SDRAM_MODE0_VAL = 0x23
		//======================================================================================================

		ldmia   r6,{r6-r11}
        str     r6,[r0]
        str     r7,[r1]
        str     r8,[r2]
        str     r9,[r3]
        str     r10,[r4]
        str     r11,[r5]

        ldr     r11,=EPXA1_SDRAM_BASE
        ldr     r0,=EPXA1_SDRAM_WIDTH
        ldr     r1,=(_SDRAM_WIDTH_W | _SDRAM_WIDTH_LK)  /* 16 bit wide */
        str     r1,[r0]

#ifdef _ALTERA_CACHEHACK
        // OK we have a minor "feature/bug" in the chip which requires
        // us to be a little clever now.  The code between the labels
        // Cache_Start and Cache_stop starts the SDRAM controller and
        // issues the pre_charge command
 
        // It must execute within one refresh period, so we must load
        // this code and the code it calls (Issue_SDRAM_Command) into
        // cache, as typical flashes have a cycle time of ~100nS If we
        // are doing byte loads that's ~400ns per instruction.  Depending
        // upon the size of the DDR the refresh period can be as low as
        // 7us (smallest we found).
        adr     r1,SDR_Cache_Start
        adr     r2,SDR_Cache_Stop       
        adr     r3,Issue_SDRAM_Command
        adr     r4,End_Issue_SDRAM_Command

SDR_Load_Code:  
        mcr     p15,0,r1,c7,c13,1
        add     r1,r1,#32
        cmp     r1,r2
        ble     SDR_Load_Code
        
SDR_Load_Code2:         
        mcr     p15,0,r3,c7,c13,1
        add     r3,r3,#32
        cmp     r3,r4
        ble     SDR_Load_Code2
#endif // _ALTERA_CACHEHACK

        // setup the timer for later, (mvn!!) counter limit = 0xfffffffe
        ldr     r4,=EPXA1_TIMER0_LIMIT
        mvn     r5, #1
        str     r5, [r4]

        ldr     r4,=_SDRAM_INIT_PR
        ldr     r5,=_SDRAM_INIT_RF
        ldr     r6,=_SDRAM_INIT_LM
        ldr     r9,=EPXA1_TIMER0_READ
        ldr     r10,=EPXA1_TIMER0_CR

        // Enable the controller by setting the EN bit in the
        // SDRAM_INIT register
        mov     r1,#_SDRAM_INIT_EN
SDR_Cache_Start:
        str     r1,[r11, #_SDRAM_INIT]

        // Short delay : 200 count
		// start timer
        mov     r3, #EPXA1_TIMER_CR_S
        str     r3, [r10]
		// read timer current timer counter value
        ldr     r3, [r9]
        add     r3,r3,#200 
short_delay:
        ldr     r8, [r9]
        cmp     r3, r8
        bgt     short_delay
		// stop timer
        mov     r3, #0
        str     r3, [r10]

        // 4. Issue "pre-charge all" command
        mov     r1,r4
        bl      Issue_SDRAM_Command

        // 5. Issue two "Refresh" Commands
        mov     r1, r5
        mov     r4,#2
SDR_RF_Command:
        bl      Issue_SDRAM_Command
        subs    r4,r4,#1
        bne     SDR_RF_Command

        // 6. Issue a "load mode register" command with the SDRAM Device's DLL being Reset
        mov     r1, r6
        bl      Issue_SDRAM_Command

SDR_Cache_Stop:
        b        10f


        //--------------------------------------------
        // Issue a command from the SDRAM controller
        // Assumes:
        // r1 is the command to be issued
        // r2 and r3 are trashed
Issue_SDRAM_Command:
        mov     r2,#_SDRAM_INIT_EN
        orr     r2,r2,r1
        str     r2,[r11, #_SDRAM_INIT]
 
        // OK chip bug, the busy bit does not work properly, so we need
        // to insert a delay of 50 SDRAM clock cycles here NB Obviously
        // this must change when either the SDRAM clock or the processor
        // clock change
        ldr     r2,=(EPXA1_CLK_BASE+_CLK_AHB1_COUNT)
        ldr     r3, [r2]
        add     r3,r3,#200 // CPU 150 MHz SDRAM 75 MHz
clock_cycles:
        ldr     r8, [r2]
        cmp     r3, r8
        bgt     clock_cycles
        mov     pc, lr
End_Issue_SDRAM_Command:
        

        // SDRAM Register Addresses
SDRAM_REGS_ADDR: 
        .long EPXA1_SDRAM_BASE+_SDRAM_TIMING1
        .long EPXA1_SDRAM_BASE+_SDRAM_TIMING2
        .long EPXA1_SDRAM_BASE+_SDRAM_CONFIG
        .long EPXA1_SDRAM_BASE+_SDRAM_REFRESH
        .long EPXA1_SDRAM_BASE+_SDRAM_ADDR
        .long EPXA1_SDRAM_BASE+_SDRAM_MODE0
        .long EPXA1_SDRAM_BASE+_SDRAM_MODE1

SDRAM_REGS_VALUE:
        .long 0x000090db /* SDRAM_TIMING1_VAL */
        .long 0x000087c0 /* SDRAM_TIMING2_VAL */
        .long 0x00000000 /* SDR */
        .long 0x000002bc /* SDRAM_REFRESH_VAL */
        .long 0x0000d980 /* SDRAM_ADDR_VAL */
        .long 0x00000023 /* SDRAM_MODE0_VAL */
        .long 0x00000000 /* SDRAM_MODE1_VAL */ // not used here
10:
#ifdef _ALTERA_CACHEHACK
        mrc     p15,0,r0,c1,c0,0
        bic     r0,r0,#0x1000              // disable ICache
        mcr     p15,0,r0,c1,c0,0
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
#endif // _ALTERA_CACHEHACK

        RAW_LED_MACRO 6	// 'G'

		// only copy the code and data from flash to RAM if it is not target for ROM -- Run from it.
		// for ROMRAM and Redboot it will first run from flash and copy to RAM and afterward run from RAM
#if defined(CYG_HAL_STARTUP_ROMRAM) || defined(CYG_HAL_STARTUP_REDBOOT)
		// FYI: epxa1_redboot.ldi
		// copy the 1. rom_vectors, 2. text(code, itself), 3. fini, 4. rodata, 5. rodata1, 6. fixup, 7. gcc_except_table, 8. data sections from the Flash to SDRAM
		// __rom_vectors_lma marks the start of the rom_vectors, text(code)... data sections in flash memory
		// __ram_data_end marks the end of the rom_vectors, text(code)... data sections 
		// __rom_vectors_lma: address of Flash memory resides after remapping -- 0x40000000
		// __rom_vectors_vma: address of rom_vectors should reside when program is run -- 0x1f40000
		// section "vectors" in the vector.S will be collected into section "rom_vectors", see arm.ld for detail
		
		// HQ TESTING
		RAW_LED_MACRO 10	// 'K'

		ldr     r0,=__rom_vectors_lma   // Relocate FLASH/ROM to SDRAM
        ldr     r1,=__rom_vectors_vma   // ram base & length
        ldr     r2,=__ram_data_end
20:     ldr     r3,[r0],#4
        str     r3,[r1],#4
        cmp     r1,r2
        bne     20b
		
		// by here still run from flash memory
		// so need to switch to SDRAM for fast correct execution 
		// so load the address of the label 30 into r0, (this would be the address of label 30 in SDRAM, address resolved by the linker
		// will be based on the section VMA
		// the branch(b,bl,bxx) command will resolve the address by pc relative(+-32MBytes) so not the same as this one.
        ldr     r0,=30f
        nop

		// directlly load the address into pc will cause execution jump to SDRAM
        mov     pc,r0
	nop
	// now we start execution from SDRAM now
30:	nop
#endif

        RAW_LED_MACRO 7	// 'H'


        // Set up a stack [for calling C code]
        ldr     r1,=__startup_stack

		// coz __startup_stack is located in the bss section and in the linker script is defined
		// in the ram and start after data section. e.g. targetting for rom bootup
		// data section: start at 0x8000
		// bss section: follows it
		// so when compile the program the label __startup_stack in bss section get addressed at some where
		// after data section(0x8000, ram), let's assumes it at 0x9000(in RAM), so 0x9000 will be the address of label
		// "__startup_stack", but in some case maybe RAM is located not from 0x00(maybe by this point memory map still
		// not setup yet, so address 0x9000 probably address somewhere not the correct address,this 0x9000 address stands
		// for from RAM start to 0x9000 location ), then adding a offset(current RAM location) to it is needed.
		//
		// for example: 
		// by this point SDRAM: 0x10000000
		// in the linker script bss section start at: 0x9000 (bss should be in ram)
		// sp should point to 0x10009000 for the location of 0x9000 in SDRAM

        ldr     r2,=EPXA1_SDRAM_PHYS_BASE
        orr     sp,r1,r2

        // Create MMU tables
        bl      hal_mmu_init

        RAW_LED_MACRO 8	// 'I'

        // Enable MMU
		// retrieve the address before memory remapping, otherwise after memory remapping the pc will point to somewhere not correct.
		ldr     r2,=10f
        ldr     r1,=MMU_Control_Init|MMU_Control_M
        mcr     MMU_CP,0,r1,MMU_Control,c0
		// branch to label 10 after memory remapping by changing the pc directly rather than branch to label.(coz memory has be remapped)
        mov     pc,r2    // Change address spaces
        nop
        nop
        nop
10:

        RAW_LED_MACRO 9	// 'J'
        .endm
        
#else // defined(CYG_HAL_STARTUP_ROM) || defined(CYG_HAL_STARTUP_ROMRAM) || defined(CYG_HAL_STARTUP_REDBOOT)
#define PLATFORM_SETUP1	// null MACRO target for RAM application.
#endif

//-----------------------------------------------------------------------------
// 0x61 -- ASCII "a"
#ifdef DEBUG
#define CYGHWR_LED_MACRO                            \
        ldr     r0,=(EPXA1_UART0_BASE+_UART_TD);	\
		ldr		r1,=(0x60+\x);						\
        str     r1,[r0];
#endif

//-----------------------------------------------------------------------------
// end of hal_platform_setup.h
#endif // CYGONCE_HAL_PLATFORM_SETUP_H
